import tmd
import view

### Defining average images

def img_diff_distance(Z1, Z2, norm=True):
    """Takes as input two images
       as exported from the gaussian kernel
       plotting function, and returns
       their absolute difference:
       diff(abs(Z1 - Z2))
    """
    if norm:
        Z1 = Z1 / Z1.max()
        Z2 = Z2 / Z2.max()

    diff = np.sum(np.abs(Z2 - Z2))

    # Normalize the difference to % of #pixels
    diff = diff / np.prod(np.shape(Z1))

    return diff

def img_diff(Z1, Z2, norm=True):
    """Takes as input two images
       as exported from the gaussian kernel
       plotting function, and returns
       their absolute difference:
       diff(abs(Z1 - Z2))
    """
    if norm:
        Z1 = Z1 / Z1.max()
        Z2 = Z2 / Z2.max()

    return Z1 - Z2


def number_of_bars(phs_list):
    '''Returns the length of each ph_diagram of a list
    '''
    return [len(p) for p in phs_list]


def weight_bars(phs_list, max_norm=None):
    '''Returns weights according to the number of branches.
    If max not defined the maximum within a group will be used.
    '''
    lengths = number_of_bars(phs_list)
    if max_norm is None:
        max_norm = np.max(lengths)
    return np.array(lengths, dtype=np.float) / max_norm


def persistence_image(ph, norm_factor=None, xlims=None, ylims=None):
    '''Create the data for the generation of the persistence image.
    If norm_factor is provided the data will be normalized based on this,
    otherwise they will be normalized to 1.
    If xlims, ylims are provided the data will be scaled accordingly.   
    '''
    from scipy import stats
    import numpy as np

    if xlims is None:
        xlims = [min(np.transpose(ph)[0]), max(np.transpose(ph)[0])]
    if ylims is None:
        ylims = [min(np.transpose(ph)[1]), max(np.transpose(ph)[1])]

    X, Y = np.mgrid[xlims[0]:xlims[1]:100j, ylims[0]:ylims[1]:100j]

    values = np.transpose(ph)
    kernel = stats.gaussian_kde(values)
    positions = np.vstack([X.ravel(), Y.ravel()])
    Z = np.reshape(kernel(positions).T, X.shape)

    if norm_factor is None:
        norm_factor = np.max(Z)

    Zn = Z / norm_factor

    return Zn


def average_ph_image(images_list):
    '''Generates a normalized average image
    from a list of images. Careful: images should be
    at the same scale (x-y) for appropriate comparison.
    '''
    average_imgs = images_list[0]

    for im in images_list[1:]:
        average_imgs = np.add(average_imgs, im)

    average_imgs = average_imgs / len(images_list)

    return average_imgs


def average_ph_image_weighted(images_list, weights):
    '''Generates a normalized average image
    from a list of images. Careful: images should be
    at the same scale (x-y) for appropriate comparison.
    '''
    average_imgs = weights[0] * images_list[0]

    for i,im in enumerate(images_list[1:]):
        average_imgs = np.add(average_imgs, weights[i] * im)

    average_imgs = average_imgs / len(images_list)

    return average_imgs / np.max(average_imgs)

def average_ph_from_list(phs_list, xlims=None, ylims=None, norm_factor=None):
    '''Generates average image from list of phs'''
    imgs = [persistence_image(p, norm_factor=norm_factor, xlims=xlims, ylims=ylims) for p in phs_list]
    return average_ph_image(imgs)


def average_weighted_ph_from_list(phs_list, xlims=None, ylims=None, norm_factor=None, max_norm=None):
    '''Generates average image from list of phs'''
    weights = weight_bars(phs_list, max_norm=max_norm)
    imgs = [persistence_image(p, norm_factor=norm_factor, xlims=xlims, ylims=ylims) for p in phs_list]
    return average_ph_image_weighted(imgs, weights=weights)


def define_limits(phs_list):
    '''Returns the x-y coordinates limits (min, max)
    for a list of persistence diagrams
    '''
    ph = view.plot.collapse(phs_list)
    xlims = [min(np.transpose(ph)[0]), max(np.transpose(ph)[0])]
    ylims = [min(np.transpose(ph)[1]), max(np.transpose(ph)[1])]

    return xlims, ylims


### Ploting functions

def plot_imgs(img, new_fig=True, subplot=111, title='', xlims=None, ylims=None, cmap=cm.jet, vmin=0, vmax=1., masked=False, threshold=0.01):
    '''Plots the gaussian kernel of the input image.
    '''
    from scipy import stats
    from view import common
    import numpy as np

    if xlims is None:
        xlims = (0,100)
    if ylims is None:
        ylims = (0,100)

    fig, ax = common.get_figure(new_fig=new_fig, subplot=subplot)

    if masked:
        img = np.ma.masked_where((threshold > np.abs(img)), img)

    cax= ax.imshow(np.rot90(img), vmin=vmin, vmax=vmax, cmap=cmap, interpolation='bilinear', extent=xlims+ylims)

    kwargs = {}

    kwargs['xlim'] = xlims
    kwargs['ylim'] = ylims
    kwargs['title'] = title

    plt.colorbar(cax)

    ax.set_aspect('equal')

    return common.plot_style(fig=fig, ax=ax, aspect='equal', **kwargs)


def plot_av(phs1, title=''):
    '''Generates and plots the average images from input phs'''
    xlims, ylims = define_limits(phs1)
    imgs1 = [persistence_image(p, xlims=xlims, ylims=ylims) for p in phs1]
    IMG = average_ph_image(imgs1)
    return plot_imgs(IMG, xlims=xlims, ylims=ylims, title=title)


def multiplot(phs1, title=''):
    '''Plots distances, average image and an example from the population'''

    xlims, ylims = define_limits(phs1)
    imgs1 = [persistence_image(p, xlims=xlims, ylims=ylims) for p in phs1]

    IMG = average_ph_image(imgs1)

    distances = [img_diff_distance(IMG, im) for im in imgs1]
    ID = np.argmax(distances)
    
    fig = plt.figure()
    ax = fig.add_subplot(1,3,1)
    ax.hist(distances, bins=20)
    plt.title(title)

    plot_imgs(IMG, xlims=xlims, ylims=ylims, new_fig=False, subplot=132)
    plot_imgs(imgs1[ID], xlims=xlims, ylims=ylims, new_fig=False, subplot=133)

    return distances


def multiplot_outliers(phs1, title=''):
    '''Plots distances, average image and an example from the population'''

    xlims, ylims = define_limits(phs1)
    imgs1 = [persistence_image(p, xlims=xlims, ylims=ylims) for p in phs1]

    IMG = average_ph_image(imgs1)

    distances = [img_diff_distance(IMG, im) for im in imgs1]

    d_mean = np.mean(distances)
    d_std = np.std(distances)
    outliers = np.where((np.array(distances) > d_mean + 2*d_std))[0]
    no_outliers = np.delete(np.arange(len(distances)), outliers)

    IMG_out = average_ph_image(np.array(imgs1)[outliers])
    IMG_without = average_ph_image(np.array(imgs1)[no_outliers])

    fig = plt.figure()
    ax = fig.add_subplot(2,2,1)
    ax.hist(distances, bins=20)
    plt.title(title)

    plot_imgs(IMG, xlims=xlims, ylims=ylims, new_fig=False, subplot=222)
    plt.title('All')
    plot_imgs(IMG_out, xlims=xlims, ylims=ylims, new_fig=False, subplot=223)
    plt.title('Outliers')
    plot_imgs(IMG_without, xlims=xlims, ylims=ylims, new_fig=False, subplot=224)
    plt.title('No_outliers')

    return distances


def outliers_distances(dist_list, n=2):
    '''Outputs the number of cells
    that are outside the n*sigma regime.
    '''
    d_mean = np.mean(dist_list)
    d_std = np.std(dist_list)

    outliers = np.where((np.array(dist_list) > d_mean + n*d_std))[0]

    return outliers, len(outliers), len(dist_list)


def representatives_distances(dist_list, n=3):
    '''Outputs the number of cells
    that are outside the n*sigma regime.
    '''
    return np.argsort(dist_list)[:n]

# Functions using the previous basics, customized for Sandra's datasets

def example_run_outliers(filename='./Female/control 4h/IPL/', title=''):
    '''Special funct'''
    pop = tmd.io.load_population(filename, tree_types={0:'basal'})

    phs1 = []
    pids = []

    for i,n in enumerate(pop.neurons):
        try:
            p = tmd.methods.get_ph_neuron(n)
            if len(p) > 4:
                phs1.append(p)
                pids.append(i)
        except:
            print n.name

    distances = multiplot_outliers(phs1, title=title)

    return distances


def example_run(filename='./Female/control 4h/IPL/', title=''):
    '''Special funct'''
    pop = tmd.io.load_population(filename, tree_types={0:'basal'})

    phs1 = []

    for i,n in enumerate(pop.neurons):
        try:
            p = tmd.methods.get_ph_neuron(n)
            if len(p) > 4:
                phs1.append(p)
        except:
            #print n.name
            pass

    distances = multiplot(phs1, title=title)

    return distances


def get_phs_clean(filename='./Female/control 4h/IPL/'):
    '''Special funct'''
    pop = tmd.io.load_population(filename, tree_types={0:'basal'})

    phs1 = []

    for i,n in enumerate(pop.neurons):
        try:
            p = tmd.methods.get_ph_neuron(n)
            if len(p) > 4:
                phs1.append(p)
        except:
            #print n.name
            pass

    return phs1


def norm_limits(ph_list):
    '''Normalizes the limits and the intensity of ph_images
    in order to compare all the ph of a list.
    '''
    # Normalization of limits
    pp = []
    for ph in ph_list:
        pp = pp + ph
    xlims, ylims = define_limits(pp)

    return xlims, ylims


def norm_intensity(ph_list):
    '''Normalizes the limits and the intensity of ph_images
    in order to compare all the ph of a list.
    '''
    # Normalization of intensity
    intensities = []
    for ph in ph_list:
        Z = average_ph_from_list(ph, norm_factor=1.0, xlims=xlims, ylims=ylims)
        intensities.append(np.max(Z))
    M = np.max(intensities)

    return M


def multiplot_comparison(ph_list, norm=True, diff=1., thresh=0.1, masked=True):

    xlims, ylims = norm_limits(ph_list)

    if norm:
        M = norm_intensity(ph_list)
    else:
        M = None

    # Generate normalized averages
    Zns = []
    for ph in ph_list:
        Z = average_ph_from_list(ph, norm_factor=M, xlims=xlims, ylims=ylims)
        Zns.append(Z)

    plot_imgs(Zns[0], new_fig=True, subplot=331, xlims=xlims, ylims=ylims)
    plot_imgs(Zns[1], new_fig=False, subplot=332, xlims=xlims, ylims=ylims)
    plot_imgs(Zns[2], new_fig=False, subplot=334, xlims=xlims, ylims=ylims)
    plot_imgs(Zns[3], new_fig=False, subplot=335, xlims=xlims, ylims=ylims)

    D1 = img_diff(Zns[0], Zns[2], norm=False)
    D2 = img_diff(Zns[1], Zns[3], norm=False)
    D3 = img_diff(Zns[0], Zns[1], norm=False)
    D4 = img_diff(Zns[2], Zns[3], norm=False)

    plot_imgs(D1, new_fig=False, subplot=337, xlims=xlims, ylims=ylims, vmin=-diff, vmax=diff, masked=masked, threshold=thresh)
    plot_imgs(D2, new_fig=False, subplot=338, xlims=xlims, ylims=ylims, vmin=-diff, vmax=diff, masked=masked, threshold=thresh)
    plot_imgs(D3, new_fig=False, subplot=333, xlims=xlims, ylims=ylims, vmin=-diff, vmax=diff, masked=masked, threshold=thresh)
    plot_imgs(D4, new_fig=False, subplot=336, xlims=xlims, ylims=ylims, vmin=-diff, vmax=diff, masked=masked, threshold=thresh)

    return Zns


def multiplot_weighted_comparison_naive(ph_list, diff=1., thresh=0.1, masked=True, title='', xlims=None, ylims=None):

    if xlims is None and ylims is None:    
        xlims, ylims = norm_limits(ph_list)

    # Generate normalized averages
    Zns = []
    for ph in ph_list:
        Z = average_weighted_ph_from_list(ph, norm_factor=None, xlims=xlims, ylims=ylims)
        Zns.append(Z)

    plot_imgs(Zns[0], new_fig=True, subplot=331, xlims=xlims, ylims=ylims, title='Control 4h')
    plot_imgs(Zns[1], new_fig=False, subplot=332, xlims=xlims, ylims=ylims, title='Control 48h')
    plot_imgs(Zns[2], new_fig=False, subplot=334, xlims=xlims, ylims=ylims, title='Naive 4h')
    plot_imgs(Zns[3], new_fig=False, subplot=335, xlims=xlims, ylims=ylims, title='Naive 48h')

    D1 = img_diff(Zns[0], Zns[2], norm=False)
    D2 = img_diff(Zns[1], Zns[3], norm=False)
    D3 = img_diff(Zns[0], Zns[1], norm=False)
    D4 = img_diff(Zns[2], Zns[3], norm=False)

    plot_imgs(D1, new_fig=False, subplot=337, xlims=xlims, ylims=ylims, vmin=-diff, vmax=diff, masked=masked, threshold=thresh, title='Control - Naive 4h')
    print 'Control - Naive 4h :', np.sum(np.abs(D1))
    plot_imgs(D2, new_fig=False, subplot=338, xlims=xlims, ylims=ylims, vmin=-diff, vmax=diff, masked=masked, threshold=thresh, title='Control - Naive 48h')
    print 'Control - Naive 48h :', np.sum(np.abs(D2))
    plot_imgs(D3, new_fig=False, subplot=333, xlims=xlims, ylims=ylims, vmin=-diff, vmax=diff, masked=masked, threshold=thresh, title='Control 4h-48h')
    print 'Control 4h-48h :', np.sum(np.abs(D3))
    plot_imgs(D4, new_fig=False, subplot=336, xlims=xlims, ylims=ylims, vmin=-diff, vmax=diff, masked=masked, threshold=thresh, title='Naive 4h-48h')
    print 'Naive 4h-48h :', np.sum(np.abs(D4))

    plt.suptitle(title)
    plt.tight_layout(True)

    return Zns



def multiplot_weighted_comparison(ph_list, diff=1., thresh=0.1, masked=True, title='', xlims=None, ylims=None):

    if xlims is None and ylims is None:    
        xlims, ylims = norm_limits(ph_list)

    # Generate normalized averages
    Zns = []
    for ph in ph_list:
        Z = average_weighted_ph_from_list(ph, norm_factor=None, xlims=xlims, ylims=ylims)
        Zns.append(Z)

    plot_imgs(Zns[0], new_fig=True, subplot=331, xlims=xlims, ylims=ylims, title='Control 4h')
    plot_imgs(Zns[1], new_fig=False, subplot=332, xlims=xlims, ylims=ylims, title='Control 48h')
    plot_imgs(Zns[2], new_fig=False, subplot=334, xlims=xlims, ylims=ylims, title='Drug 4h')
    plot_imgs(Zns[3], new_fig=False, subplot=335, xlims=xlims, ylims=ylims, title='Drug 48h')

    D1 = img_diff(Zns[0], Zns[2], norm=False)
    D2 = img_diff(Zns[1], Zns[3], norm=False)
    D3 = img_diff(Zns[0], Zns[1], norm=False)
    D4 = img_diff(Zns[2], Zns[3], norm=False)

    plot_imgs(D1, new_fig=False, subplot=337, xlims=xlims, ylims=ylims, vmin=-diff, vmax=diff, masked=masked, threshold=thresh, title='Control - Drug 4h')
    print 'Control - Drug 4h :', np.sum(np.abs(D1))
    plot_imgs(D2, new_fig=False, subplot=338, xlims=xlims, ylims=ylims, vmin=-diff, vmax=diff, masked=masked, threshold=thresh, title='Control - Drug 48h')
    print 'Control - Drug 48h :', np.sum(np.abs(D2))
    plot_imgs(D3, new_fig=False, subplot=333, xlims=xlims, ylims=ylims, vmin=-diff, vmax=diff, masked=masked, threshold=thresh, title='Control 4h-48h')
    print 'Control 4h-48h :', np.sum(np.abs(D3))
    plot_imgs(D4, new_fig=False, subplot=336, xlims=xlims, ylims=ylims, vmin=-diff, vmax=diff, masked=masked, threshold=thresh, title='Drug 4h-48h')
    print 'Drug 4h-48h :', np.sum(np.abs(D4))

    plt.suptitle(title)
    plt.tight_layout(True)

    return Zns


def multiplot_weighted_comparison_male_female(ph_list, diff=1., thresh=0.1, masked=True, title='', xlims=None, ylims=None):

    if xlims is None and ylims is None:    
        xlims, ylims = norm_limits(ph_list)

    # Generate normalized averages
    Zns = []
    for ph in ph_list:
        Z = average_weighted_ph_from_list(ph, norm_factor=None, xlims=xlims, ylims=ylims)
        Zns.append(Z)

    plot_imgs(Zns[0], new_fig=True, subplot=331, xlims=xlims, ylims=ylims, title='Control 4h')
    plot_imgs(Zns[1], new_fig=False, subplot=332, xlims=xlims, ylims=ylims, title='Control 48h')
    plot_imgs(Zns[2], new_fig=False, subplot=334, xlims=xlims, ylims=ylims, title='Drug 4h')
    plot_imgs(Zns[3], new_fig=False, subplot=335, xlims=xlims, ylims=ylims, title='Drug 48h')

    D1 = img_diff(Zns[0], Zns[2], norm=False)
    D2 = img_diff(Zns[1], Zns[3], norm=False)
    D3 = img_diff(Zns[0], Zns[1], norm=False)
    D4 = img_diff(Zns[2], Zns[3], norm=False)

    plot_imgs(D1, new_fig=False, subplot=337, xlims=xlims, ylims=ylims, vmin=-diff, vmax=diff, masked=masked, threshold=thresh, title='Control - Drug 4h')
    print 'Control - Drug 4h :', np.sum(np.abs(D1))
    plot_imgs(D2, new_fig=False, subplot=338, xlims=xlims, ylims=ylims, vmin=-diff, vmax=diff, masked=masked, threshold=thresh, title='Control - Drug 48h')
    print 'Control - Drug 48h :', np.sum(np.abs(D2))
    plot_imgs(D3, new_fig=False, subplot=333, xlims=xlims, ylims=ylims, vmin=-diff, vmax=diff, masked=masked, threshold=thresh, title='Control 4h-48h')
    print 'Control 4h-48h :', np.sum(np.abs(D3))
    plot_imgs(D4, new_fig=False, subplot=336, xlims=xlims, ylims=ylims, vmin=-diff, vmax=diff, masked=masked, threshold=thresh, title='Drug 4h-48h')
    print 'Drug 4h-48h :', np.sum(np.abs(D4))

    plt.suptitle(title)
    plt.tight_layout(True)

    return Zns


def distance_number_of_cells(ph_list, step_size=10, samples=10, xlims=None, ylims=None, title=''):
    #import seaborn
    min_max_size = np.min([len(p) for p in ph_list])
    intervals = np.linspace(step_size, min_max_size, (min_max_size-step_size)/step_size)

    distancesCD4 = []
    distancesCD4std = []
    distancesCD48 = []
    distancesCD48std = []

    for i in intervals:

        d4 = []
        d48 = []

        for s in np.arange(samples):
            Zns = []
            for ph in ph_list:
                ph_random_indices = random.choice(np.arange(len(ph)), int(i), replace=False)
                ph_random = np.array(ph)[ph_random_indices]
                Z = average_weighted_ph_from_list(ph_random, norm_factor=None, xlims=xlims, ylims=ylims)
                Zns.append(Z)

            DiffCD4 = np.sum(np.abs(img_diff(Zns[0], Zns[1], norm=False)))
            DiffCD48 = np.sum(np.abs(img_diff(Zns[0], Zns[2], norm=False)))

            d4.append(DiffCD4/ (DiffCD4+DiffCD48))
            d48.append(DiffCD48/ (DiffCD4+DiffCD48))

        distancesCD4.append(np.mean(d4))
        distancesCD48.append(np.mean(d48))
        distancesCD4std.append(np.std(d4))
        distancesCD48std.append(np.std(d48))

    fig = plt.figure(figsize=(15,10))
    plt.plot(intervals, distancesCD4, c='b', label='Dist: C-D4')
    plt.plot(intervals, distancesCD48, c='r', label='Dist: C-D48')

    plt.fill_between(intervals, np.subtract(distancesCD4, distancesCD4std), np.add(distancesCD4, distancesCD4std), color='b', alpha=0.2)
    plt.fill_between(intervals, np.subtract(distancesCD48, distancesCD48std), np.add(distancesCD48, distancesCD48std), color='r', alpha=0.2)

    plt.xlabel('Number of cells', fontsize=18)
    plt.ylabel('Normalized distance', fontsize=18)
    plt.legend()
    plt.title(title)

    return intervals, distancesCD4, distancesCD48, distancesCD4std, distancesCD48std


def check_input_data(filename):

    import os
    L = os.listdir(filename)

    counter_process = 0
    counter_load = 0
    
    for c in L:
        try:
            n = tmd.io.load_neuron(filename+c, tree_types={0:'basal'})
            try:
                p = tmd.methods.get_ph_neuron(n)
            except:
                counter_process = counter_process + 1
        except:
            counter_load = counter_load + 1
            # print c

    return len(L), counter_load, counter_process




'''Test tmd.topology.analysis'''
from nose import tools as nt
import numpy as np
from numpy.testing import assert_equal, assert_array_equal, assert_array_almost_equal, assert_almost_equal
from tmd.Topology import analysis
import os

_path = os.path.dirname(os.path.abspath(__file__))
DATA_PATH = os.path.join(_path, 'data')

# Filenames for testing
sample_ph_0_file = os.path.join(DATA_PATH, 'sample_ph_0.txt')
sample_ph_1_file = os.path.join(DATA_PATH, 'sample_ph_1.txt')
neuron_ph_1_file = os.path.join(DATA_PATH, 'neuron_ph_1.txt')
neuron_ph_2_file = os.path.join(DATA_PATH, 'neuron_ph_2.txt')

sample_data_0 = np.array([[10, 8], [9, 8], [9, 8]])
sample_data_1 = np.array([[10, 7], [9, 8], [9, 8]])

def test_load_file():
    f0 = analysis.load_file(sample_ph_0_file)
    f1 = analysis.load_file(sample_ph_1_file)
    nt.ok_(np.allclose(f0, np.array([[ 12.24744871,  11.18033989],
                                     [ 11.18033989,  10.        ],
                                     [ 12.24744871,  0.        ]])))
    nt.ok_(np.allclose(f1, np.array([[ 11.18033989,  10.        ],
                                     [ 11.18033989,  0.        ]])))

def test_histogram_horizontal():
    bins, data = analysis.histogram_horizontal(sample_data_0, num_bins=11, min_bin=8.0, max_bin=10.0)
    nt.ok_(np.allclose(bins, np.array([ 8., 8.2, 8.4, 8.6, 8.8, 9., 9.2, 9.4, 9.6, 9.8, 10. ])))
    nt.ok_(np.allclose(data, np.array([ 3.,  3.,  3.,  3.,  1.,  3.,  1.,  1.,  1.,  1.])))
    bins, data = analysis.histogram_horizontal(sample_data_0, num_bins=10)
    nt.ok_(np.allclose(bins, np.array([ 8.        , 8.22222222,   8.44444444,   8.66666667,
                                        8.88888889, 9.11111111,   9.33333333,   9.55555556,
                                        9.77777778, 10.])))
    nt.ok_(np.allclose(data, np.array([ 3.,  3.,  3.,  3.,  3.,  1.,  1.,  1.,  1.])))
    bins, data = analysis.histogram_horizontal(sample_data_0, num_bins=4, min_bin=7., max_bin=10.)
    nt.ok_(np.allclose(bins, np.array([ 7.,   8.,   9.,  10.])))
    nt.ok_(np.allclose(data, np.array([ 0.,  3.,  1.])))
    bins, data = analysis.histogram_horizontal(sample_data_1, num_bins=4, min_bin=7., max_bin=10.)
    nt.ok_(np.allclose(bins, np.array([ 7.,   8.,   9.,  10.])))
    nt.ok_(np.allclose(data, np.array([ 1.,  3.,  1.])))

def test_histogram_stepped():
    bins, data = analysis.histogram_stepped(sample_data_0)
    nt.ok_(np.allclose(bins, np.array([ 8,  9, 10])))
    nt.ok_(np.allclose(data, np.array([ 3.,  1.])))
    bins, data = analysis.histogram_stepped(sample_data_1)
    nt.ok_(np.allclose(bins, np.array([ 7,  8,  9, 10])))
    nt.ok_(np.allclose(data, np.array([ 3.,  3.,  1.])))

def test_distance_stepped():
    f0 = analysis.load_file(sample_ph_0_file)
    f1 = analysis.load_file(sample_ph_1_file)
    nt.ok_(analysis.distance_stepped(f0, f0) == 0.0)
    nt.ok_(analysis.distance_stepped(f0, f0, order=1) == 0.0)
    nt.ok_(analysis.distance_stepped(f0, f0, order=2) == 0.0)
    nt.ok_(np.allclose(analysis.distance_stepped(f0, f1), 34.01795854515))
    nt.ok_(np.allclose(analysis.distance_stepped(f0, f1, order=1), 34.01795854515))
    nt.ok_(np.allclose(analysis.distance_stepped(f0, f1, order=2), 25.71017265966))

def test_distance_horizontal():
    nt.ok_(analysis.distance_horizontal(sample_data_0, sample_data_0)==0.0)
    nt.ok_(analysis.distance_horizontal(sample_data_0, sample_data_1, bins=4) == 2.0)

def test_distance_horizontal_unnormed():
    nt.ok_(analysis.distance_horizontal_unnormed(sample_data_0, sample_data_0)==0.0)
    nt.ok_(analysis.distance_horizontal_unnormed(sample_data_0, sample_data_1, bins=4) == 1.0)


def _ph_list():

    return [
                [
                    [16.90, 6.68, 0.1, 0.1, 0.1, 0.1],
                    [10.52, 5.98, 0.1, 0.1, 0.1, 0.1],
                    [74.11, 0.00, 0.1, 0.1, 0.1, 0.1]
                ],
                [
                    [3.010, 1.58, 0.1, 0.1, 0.1, 0.1],
                    [15.22, 0.18, 0.1, 0.1, 0.1, 0.1],
                    [60.48, 0.00, 0.1, 0.1, 0.1, 0.1]
                ],
                [
                    [9.78, 5.30, 0.1, 0.1, 0.1, 0.1],
                    [7.66, 1.60, 0.1, 0.1, 0.1, 0.1],
                    [24.0, 0.00, 0.1, 0.1, 0.1, 0.1]
                ],
                [
                    [6.05, 2.01, 0.1, 0.1, 0.1, 0.1],
                    [3.91, 1.41, 0.1, 0.1, 0.1, 0.1],
                    [8.05, 0.00, 0.1, 0.1, 0.1, 0.1]
                ],
                [
                    [2.78, 0.87, 0.1, 0.1, 0.1, 0.1],
                    [6.12, 0.21, 0.1, 0.1, 0.1, 0.1],
                    [21.2, 0.00, 0.1, 0.1, 0.1, 0.1]
                ],
                [
                    [4.99, 4.06, 0.1, 0.1, 0.1, 0.1],
                    [4.38, 2.92, 0.1, 0.1, 0.1, 0.1],
                    [6.79, 0.00, 0.1, 0.1, 0.1, 0.1]
                ],
                [
                    [4.99, 4.06, 0.1, 0.1, 0.1, 0.1],
                    [4.38, 2.92, 0.1, 0.1, 0.1, 0.1],
                    [4.79, 0.00, 0.1, 0.1, 0.1, 0.1]
                ]
    ]

def test_closest_ph__reasonable_target_extent():

    ph_list = _ph_list()

    target_extent = 6.0

    closest_index = analysis.closest_ph(ph_list, target_extent, method='from_above')
    assert_equal(closest_index, 5)

    closest_index = analysis.closest_ph(ph_list, target_extent, method='from_below')
    assert_equal(closest_index, 6)

    closest_index = analysis.closest_ph(ph_list, target_extent, method='nearest')
    assert_equal(closest_index, 5)


def test_closest_ph__very_big_target_extent():

    ph_list = _ph_list()

    target_extent = 100.

    closest_index = analysis.closest_ph(ph_list, target_extent, method='from_above')
    assert_equal(closest_index, 0)

    closest_index = analysis.closest_ph(ph_list, target_extent, method='from_below')
    assert_equal(closest_index, 0)

    closest_index = analysis.closest_ph(ph_list, target_extent, method='nearest')
    assert_equal(closest_index, 0)


def test_closest_ph__very_small_target_extent():

    ph_list = _ph_list()

    target_extent = 2.0

    closest_index = analysis.closest_ph(ph_list, target_extent, method='from_above')
    assert_equal(closest_index, 6)

    closest_index = analysis.closest_ph(ph_list, target_extent, method='from_below')
    assert_equal(closest_index, 6)

    closest_index = analysis.closest_ph(ph_list, target_extent, method='nearest')
    assert_equal(closest_index, 6)


def test_closest_ph__exact_match_target_extent():

    ph_list = _ph_list()

    target_extent = 24.0

    closest_index = analysis.closest_ph(ph_list, target_extent, method='from_above')
    assert_equal(closest_index, 2)

    closest_index = analysis.closest_ph(ph_list, target_extent, method='from_below')
    assert_equal(closest_index, 2)

    closest_index = analysis.closest_ph(ph_list, target_extent, method='nearest')
    assert_equal(closest_index, 2)


def test_apical_point():
    p1 = analysis.load_file(neuron_ph_1_file)
    p2 = analysis.load_file(neuron_ph_2_file)
    ap1 = analysis.find_apical_point_distance(p1)
    ap2 = analysis.find_apical_point_distance(p2)
    assert_almost_equal(ap1, 413.2151457659, decimal=5)
    assert_almost_equal(ap2, 335.8844214625, decimal=5)


def test_apical_point_smoothed():
    p1 = analysis.load_file(neuron_ph_1_file)
    p2 = analysis.load_file(neuron_ph_2_file)
    ap1 = analysis.find_apical_point_distance_smoothed(p1, threshold=0.1)
    ap2 = analysis.find_apical_point_distance_smoothed(p2, threshold=0.1)
    assert_almost_equal(ap1, 355.1980697442, decimal=5)
    assert_almost_equal(ap2, 199.5273018308, decimal=5)



'''
tmd Topology analysis algorithms implementation
'''
# pylint: disable=invalid-slice-index
import copy
import math
from itertools import chain
import numpy as np
from numpy.linalg import norm
from scipy.spatial.distance import cdist
from scipy import stats
from .statistics import get_lengths


def collapse(ph_list):
    '''Collapses a list of ph diagrams
       into a single instance for plotting.
    '''
    return [list(pi) for p in ph_list for pi in p]


def sort_ph(ph):
    """
    Sorts barcode according to decreasing length of bars.
    """
    return np.array(ph)[np.argsort([p[0] - p[1] for p in ph])].tolist()


def closest_ph(ph_list, target_extent, method='from_above'):
    """
    Returns the index of the persistent homology in the ph_list that has the maximum extent
    which is closer to the target_extent according to the selected method.

    method:
        from_above: smallest maximum extent that is greater or equal than target_extent
        from_below: biggest maximum extent that is smaller or equal than target_extent
        nearest: closest by absolute value
    """
    n_bars = len(ph_list)
    max_extents = np.asarray([max(get_lengths(ph)) for ph in ph_list])

    sorted_indices = np.argsort(max_extents, kind='mergesort')
    sorted_extents = max_extents[sorted_indices]

    if method == 'from_above':

        above = np.searchsorted(sorted_extents, target_extent, side='right')

        # if target extent is close to current one, return this instead
        if above >= 1 and np.isclose(sorted_extents[above - 1], target_extent):
            closest_index = above - 1
        else:
            closest_index = above

        closest_index = np.clip(closest_index, 0, n_bars - 1)

    elif method == 'from_below':

        below = np.searchsorted(sorted_extents, target_extent, side='left')

        # if target extent is close to current one, return this instead
        if below < n_bars and np.isclose(sorted_extents[below], target_extent):
            closest_index = below
        else:
            closest_index = below - 1

        closest_index = np.clip(closest_index, 0, n_bars - 1)

    elif method == 'nearest':

        below = np.searchsorted(sorted_extents, target_extent, side="left")
        pos = np.clip(below, 0, n_bars - 2)

        closest_index = \
            min((pos, pos + 1), key=lambda i: abs(sorted_extents[i] - target_extent))

    else:
        raise TypeError('Unknown method {} for closest_ph'.format(method))

    return sorted_indices[closest_index]


def load_file(filename, delimiter=' '):
    """Load PH file in a np.array
    """
    f = open(filename, 'r')
    ph = np.array([np.array(line.split(delimiter), dtype=np.float) for line in f])
    f.close()
    return ph


def get_limits(phs_list, coll=True):
    '''Returns the x-y coordinates limits (min, max)
    for a list of persistence diagrams
    '''
    if coll:
        ph = collapse(phs_list)
    else:
        ph = copy.deepcopy(phs_list)
    xlims = [min(np.transpose(ph)[0]), max(np.transpose(ph)[0])]
    ylims = [min(np.transpose(ph)[1]), max(np.transpose(ph)[1])]
    return xlims, ylims


def get_persistence_image_data(ph, norm_factor=None, xlims=None, ylims=None, bw_method=None):
    '''Create the data for the generation of the persistence image.
    ph: persistence diagram
    norm_factor: persistence image data are normalized according to this.
        If norm_factor is provided the data will be normalized based on this,
        otherwise they will be normalized to 1.
    xlims, ylims: the image limits on x-y axes.
        If xlims, ylims are provided the data will be scaled accordingly.
    bw_method: The method used to calculate the estimator bandwidth for the gaussian_kde.
    '''
    if xlims is None or xlims is None:
        xlims, ylims = get_limits(ph, coll=False)

    X, Y = np.mgrid[xlims[0]:xlims[1]:100j, ylims[0]:ylims[1]:100j]

    values = np.transpose(ph)
    kernel = stats.gaussian_kde(values, bw_method=bw_method)
    positions = np.vstack([X.ravel(), Y.ravel()])
    Z = np.reshape(kernel(positions).T, X.shape)

    if norm_factor is None:
        norm_factor = np.max(Z)

    return Z / norm_factor


def get_image_diff_data(Z1, Z2, normalized=True):
    """Takes as input two images
       as exported from the gaussian kernel
       plotting function, and returns
       their difference: diff(Z1 - Z2)
    """
    if normalized:
        Z1 = Z1 / Z1.max()
        Z2 = Z2 / Z2.max()
    return Z1 - Z2


def get_image_add_data(Z1, Z2, normalized=True):
    """Takes as input two images
       as exported from the gaussian kernel
       plotting function, and returns
       their sum: add(Z1 - Z2)
    """
    if normalized:
        Z1 = Z1 / Z1.max()
        Z2 = Z2 / Z2.max()
    return Z1 + Z2


def histogram_horizontal(ph, num_bins=100, min_bin=None, max_bin=None):
    """Calculate how many barcode lines are found in each bin.
    """
    ph1 = [p[:2] for p in ph]  # simplify to ensure ph corresponds to 2d barcode

    if min_bin is None:
        min_bin = np.min(ph1)
    if max_bin is None:
        max_bin = np.max(ph1)

    bins = np.linspace(min_bin, max_bin, num_bins, dtype=float)
    binsize = (max_bin - min_bin) / (num_bins - 1.)
    results = np.zeros(num_bins - 1)

    for p in ph1:
        ph_decompose = np.linspace(np.min(p), np.max(p),
                                   math.ceil((np.max(p) - np.min(p)) / binsize),
                                   dtype=float)

        bin_ph = np.histogram(ph_decompose, bins=bins)[0]
        results = np.add(results, bin_ph)

    return bins, results


def histogram_stepped(ph1):
    '''Calculate step distance of ph data'''
    bins = np.unique(list(chain(*ph1)))
    results = np.zeros(len(bins) - 1)

    for ph in ph1:
        for it, _ in enumerate(bins[:-1]):
            if min(ph) <= bins[it + 1] and max(ph) > bins[it]:
                results[it] = results[it] + 1

    return bins, results


def barcode_bin_centers(ph, num_bins=100, min_bin=None, max_bin=None):
    """Calculate how many barcode lines are found in each bin.
       Returns the bin centers and the number of bars that fall within a bin.
    """
    ph_2D = [p[:2] for p in ph]  # simplify to ensure ph corresponds to 2d barcode
    if min_bin is None:
        min_bin = np.min(ph_2D)
    if max_bin is None:
        max_bin = np.max(ph_2D)
    bins = np.linspace(min_bin, max_bin, num_bins, dtype=float)
    bin_centers = 0.5 * (bins[:-1] + bins[1:])

    data = [bin_centers[i] for p in ph_2D for i in range(num_bins - 1)
            if np.max(p) > bins[i + 1] and np.min(p) < bins[i]]
    # data = []
    # for i in range(num_bins - 1):
    #    for p in ph_2D:
    #        if np.max(p) > bins[i + 1] and np.min(p) < bins[i]:
    #            data.append(bin_centers[i])
    return bin_centers, data


def distance_stepped(ph1, ph2, order=1):
    '''Calculate step distance difference between two ph'''
    bins1 = np.unique(list(chain(*ph1)))
    bins2 = np.unique(list(chain(*ph2)))
    bins = np.unique(np.append(bins1, bins2))
    results1 = np.zeros(len(bins) - 1)
    results2 = np.zeros(len(bins) - 1)

    for ph in ph1:
        for it, _ in enumerate(bins[:-1]):
            if min(ph) <= bins[it + 1] and max(ph) > bins[it]:
                results1[it] = results1[it] + 1

    for ph in ph2:
        for it, _ in enumerate(bins[:-1]):
            if min(ph) <= bins[it + 1] and max(ph) > bins[it]:
                results2[it] = results2[it] + 1

    return norm(np.abs(np.subtract(results1, results2)) * (bins[1:] + bins[:-1]) / 2, order)


def distance_horizontal(ph1, ph2, normalized=True, bins=100):
    """Calculate distance between two ph diagrams.
       Distance definition:
    """
    _, data_1 = histogram_horizontal(ph1, num_bins=bins)
    _, data_2 = histogram_horizontal(ph2, num_bins=bins)
    return norm(np.abs(np.subtract(data_1, data_2)), normalized)


def distance_horizontal_unnormed(ph1, ph2, normalized=True, bins=100):
    """Calculate unnormed distance between two ph diagrams.
    """
    maxb = np.max([np.max(ph1), np.max(ph2)])
    minb = np.min([np.min(ph1), np.min(ph2)])
    _, results1 = histogram_horizontal(ph1, num_bins=bins, min_bin=minb, max_bin=maxb)
    _, results2 = histogram_horizontal(ph2, num_bins=bins, min_bin=minb, max_bin=maxb)
    return norm(np.abs(np.subtract(results1, results2)), normalized)


def distance_persistence_image(ph1, ph2, xlims=None, ylims=None):
    """Computes the absolute difference of the respective persistence images
    """
    p1 = get_persistence_image_data(ph1, xlims=xlims, ylims=ylims)
    p2 = get_persistence_image_data(ph2, xlims=xlims, ylims=ylims)
    return np.sum(np.abs(get_image_diff_data(p1, p2)))


def get_average_persistence_image(ph_list, xlims=None, ylims=None,
                                  norm_factor=None, weighted=False):
    '''Plots the gaussian kernel of a population of cells
       as an average of the ph diagrams that are given.
    '''
    im_av = False
    k = 1
    if weighted:
        weights = [len(p) for p in ph_list]
        weights = np.array(weights, dtype=np.float) / np.max(weights)
    else:
        weights = [1 for _ in ph_list]

    for weight, ph in zip(weights, ph_list):
        if not isinstance(im_av, np.ndarray):
            try:
                im = get_persistence_image_data(ph, norm_factor=norm_factor,
                                                xlims=xlims, ylims=ylims)
                if not np.isnan(np.sum(im)):
                    im_av = weight * im
            except BaseException:  # pylint: disable=broad-except
                pass
        else:
            try:
                im = get_persistence_image_data(ph, norm_factor=norm_factor,
                                                xlims=xlims, ylims=ylims)
                if not np.isnan(np.sum(im)):
                    im_av = np.add(im_av, weight * im)
                    k = k + 1
            except BaseException:  # pylint: disable=broad-except
                pass
    return im_av / k


def find_apical_point_distance(ph):
    '''
    Finds the apical distance (measured in distance from soma)
    based on the variation of the barcode.
    '''
    # Computation of number of components within the barcode
    # as the number of bars with at least max length / 2
    lengths = get_lengths(ph)
    num_components = len(np.where(np.array(lengths) >= max(lengths) / 2.)[0])
    # Separate the barcode into sufficiently many bins
    n_bins, counts = histogram_horizontal(ph, num_bins=3 * len(ph))
    # Compute derivatives
    der1 = counts[1:] - counts[:-1]  # first derivative
    der2 = der1[1:] - der1[:-1]  # second derivative
    # Find all points that take minimum value, defined as the number of components,
    # and have the first derivative zero == no variation
    inters = np.intersect1d(np.where(counts == num_components)[0], np.where(der1 == 0)[0])
    # Find all points that are also below a positive second derivative
    # The definition of how positive the second derivative should be is arbitrary,
    # but it is the only value that works nicely for cortical cells
    try:
        best_all = inters[np.where(inters <= np.max(np.where(der2 > len(n_bins) / 100)[0]))]
    except ValueError:
        return 0.0

    if len(best_all) == 0 or n_bins[np.max(best_all)] == 0:
        return np.inf
    return n_bins[np.max(best_all)]


def find_apical_point_distance_smoothed(ph, threshold=0.1):
    '''
    Finds the apical distance (measured in distance from soma)
    based on the variation of the barcode.
    This algorithm always computes a distance, even if
    there is no obvious apical point.
    Threshold corresponds to percent of minimum derivative variation
    that is used to select the minima.
    '''
    bin_centers, data = barcode_bin_centers(ph, num_bins=100)

    # Gaussian kernel to smooth distribution of bars
    kde = stats.gaussian_kde(data)
    minimas = []
    while len(minimas) == 0:
        # Compute first derivative
        der1 = np.gradient(kde(bin_centers))
        # Compute second derivative
        der2 = np.gradient(der1)
        # Compute minima of distribution
        minimas = np.where(abs(der1) < threshold * np.max(abs(der1)))[0]
        minimas = minimas[der2[minimas] > 0]
        threshold *= 2.  # if threshold was too small, increase and retry
    return bin_centers[minimas[0]]


def _symmetric(p):
    '''Returns the symmetric point of a PD point on the diagonal
    '''
    return [(p[0] + p[1]) / 2., (p[0] + p[1]) / 2]


def matching_munkress_modified(p1, p2, use_diag=True):
    '''Returns a list of matching components
       and the corresponding distance between
       the two input diagrams
    '''
    import munkres  # pylint: disable=import-outside-toplevel
    if use_diag:
        p1_enh = p1 + [_symmetric(i) for i in p2]
        p2_enh = p2 + [_symmetric(i) for i in p1]
    else:
        p1_enh = p1
        p2_enh = p2

    D = cdist(p1_enh, p2_enh)
    m = munkres.Munkres()
    indices = m.compute(np.copy(D))
    ssum = np.sum([D[i][j] for (i, j) in indices])

    return indices, ssum



"""Plotting functions of tmd"""

import numpy as np
from matplotlib import pylab as plt
from tmd.view import common as cm
from tmd.Topology import analysis
from tmd.view.common import jet_map
from tmd.Topology.statistics import transform_ph_to_length


def barcode(ph, new_fig=True, subplot=False, color='b', linewidth=1.2, **kwargs):
    """
    Generates a 2d figure (barcode) of the persistent homology
    of a tree as it has been computed by
    Topology.get_persistent_homology method.
    """
    # Initialization of matplotlib figure and axes.
    fig, ax = cm.get_figure(new_fig=new_fig, subplot=subplot)
    ph_sort = analysis.sort_ph(ph)

    for ip, p in enumerate(ph_sort):
        ax.plot(p[:2], [ip, ip], c=color, linewidth=linewidth)

    kwargs['title'] = kwargs.get('title', 'Persistence barcode')
    kwargs['xlabel'] = kwargs.get('xlabel', 'Lifetime: radial distance from soma')

    plt.ylim([-1, len(ph_sort)])
    return cm.plot_style(fig=fig, ax=ax, **kwargs)


def barcode_enhanced(ph, new_fig=True, subplot=False, linewidth=1.2,
                     valID=2, cmap=jet_map, **kwargs):
    """
    Generates a 2d figure (barcode) of the persistent homology
    of a tree enhanced by a parameter encodes in ph[valID]
    """
    # Initialization of matplotlib figure and axes.
    fig, ax = cm.get_figure(new_fig=new_fig, subplot=subplot)
    val_max = np.max(ph, axis=0)[valID]

    # Hack for colorbar creation
    Z = [[0, 0], [0, 0]]
    levels = np.linspace(0.0, val_max, 200)
    CS3 = plt.contourf(Z, levels, cmap=cmap)

    def sort_ph_enhanced(ph, valID):
        """
        Sorts barcode according to length.
        """
        ph_sort = [p[:valID + 1] + [np.abs(p[0] - p[1])] for p in ph]
        ph_sort.sort(key=lambda x: x[valID + 1])
        return ph_sort

    ph_sort = sort_ph_enhanced(ph, valID)

    for ip, p in enumerate(ph_sort):
        ax.plot(p[:2], [ip, ip], c=cmap(p[valID] / val_max), linewidth=linewidth)

    kwargs['title'] = kwargs.get('title', 'Barcode of p.h.')
    kwargs['xlabel'] = kwargs.get('xlabel', 'Lifetime')
    plt.ylim([-1, len(ph_sort)])
    plt.colorbar(CS3)

    return cm.plot_style(fig=fig, ax=ax, **kwargs)


def diagram(ph, new_fig=True, subplot=False, color='b', alpha=1.0,
            edgecolors='black', s=30, **kwargs):
    """
    Generates a 2d figure (ph diagram) of the persistent homology of a tree.
    """
    # Initialization of matplotlib figure and axes.
    fig, ax = cm.get_figure(new_fig=new_fig, subplot=subplot)

    bounds_max = np.max(np.max(ph))
    bounds_min = np.min(np.min(ph))
    plt.plot([bounds_min, bounds_max], [bounds_min, bounds_max], c='black')

    ax.scatter(np.array(ph)[:, 0], np.array(ph)[:, 1], c=color, alpha=alpha,
               edgecolors=edgecolors, s=s)

    kwargs['title'] = kwargs.get('title', 'Persistence diagram')
    kwargs['xlabel'] = kwargs.get('xlabel', 'End radial distance from soma')
    kwargs['ylabel'] = kwargs.get('ylabel', 'Start radial distance from soma')

    return cm.plot_style(fig=fig, ax=ax, **kwargs)


def diagram_enhanced(ph, new_fig=True, subplot=False, alpha=1.0, valID=2,
                     cmap=jet_map, edgecolors='black', s=30, **kwargs):
    """
    Generates a 2d figure (diagram) of the persistent homology
    of a tree enhanced by a parameter encodes in ph[valID]
    """
    # Initialization of matplotlib figure and axes.
    fig, ax = cm.get_figure(new_fig=new_fig, subplot=subplot)
    val_max = np.max(ph, axis=0)[valID]

    # Hack for colorbar creation
    Z = [[0, 0], [0, 0]]
    levels = np.linspace(0.0, val_max, 200)
    CS3 = plt.contourf(Z, levels, cmap=cmap)

    def sort_ph_enhanced(ph, valID):
        """
        Sorts barcode according to length.
        """
        ph_sort = [p[:valID + 1] + [np.abs(p[0] - p[1])] for p in ph]
        ph_sort.sort(key=lambda x: x[valID + 1])
        return ph_sort

    ph_sort = sort_ph_enhanced(ph, valID)

    bounds_max = np.max(np.max(ph_sort))
    bounds_min = np.min(np.min(ph_sort))
    plt.plot([bounds_min, bounds_max], [bounds_min, bounds_max], c='black')

    colors = [cmap(p[valID] / val_max) for p in ph_sort]

    ax.scatter(np.array(ph_sort)[:, 0], np.array(ph_sort)[:, 1],
               c=colors, alpha=alpha, edgecolors=edgecolors, s=s)

    kwargs['title'] = kwargs.get('title', 'Persistence diagram')
    kwargs['xlabel'] = kwargs.get('xlabel', 'End radial distance from soma')
    kwargs['ylabel'] = kwargs.get('ylabel', 'Start radial distance from soma')

    plt.ylim([-1, len(ph_sort)])
    plt.colorbar(CS3)

    return cm.plot_style(fig=fig, ax=ax, **kwargs)


def persistence_image(ph, new_fig=True, subplot=111, xlims=None, ylims=None,
                      masked=False, colorbar=False, norm_factor=None, threshold=0.01,
                      vmin=None, vmax=None, cmap=jet_map, bw_method=None, **kwargs):
    '''Plots the gaussian kernel
       of the ph diagram that is given.
    '''
    if xlims is None or xlims is None:
        xlims, ylims = analysis.get_limits(ph, coll=False)

    # pylint: disable=unexpected-keyword-arg
    Zn = analysis.get_persistence_image_data(ph, norm_factor=norm_factor, bw_method=bw_method,
                                             xlims=xlims, ylims=ylims)
    fig, ax = cm.get_figure(new_fig=new_fig, subplot=subplot)

    if masked:
        Zn = np.ma.masked_where((threshold > Zn), Zn)

    cax = ax.imshow(np.rot90(Zn), vmin=vmin, vmax=vmax, cmap=cmap,
                    interpolation='bilinear', extent=xlims + ylims)

    if colorbar:
        plt.colorbar(cax)

    kwargs['xlim'] = xlims
    kwargs['ylim'] = ylims
    kwargs['title'] = kwargs.get('title', 'Persistence image')
    kwargs['xlabel'] = kwargs.get('xlabel', 'End radial distance from soma')
    kwargs['ylabel'] = kwargs.get('ylabel', 'Start radial distance from soma')

    return Zn, cm.plot_style(fig=fig, ax=ax, **kwargs)


def persistence_image_diff(Z1, Z2, new_fig=True, subplot=111, xlims=None, ylims=None,
                           norm=True, vmin=-1., vmax=1., cmap=jet_map, **kwargs):
    """Takes as input two images as exported from the gaussian kernel
       plotting function, and plots their difference.
    """
    if xlims is None or xlims is None:
        xlims, ylims = ((0, 100), (0, 100))

    difference = analysis.get_image_diff_data(Z1, Z2, normalized=norm)
    fig, ax = cm.get_figure(new_fig=new_fig, subplot=subplot)
    ax.imshow(np.rot90(difference), vmin=vmin, vmax=vmax, cmap=cmap,
              interpolation='bilinear', extent=xlims + ylims)

    kwargs['xlim'] = xlims
    kwargs['ylim'] = ylims
    return cm.plot_style(fig=fig, ax=ax, **kwargs)


def persistence_image_add(Z2, Z1, new_fig=True, subplot=111, xlims=None, ylims=None,
                          norm=True, vmin=0, vmax=2., cmap=jet_map, **kwargs):
    """Takes as input two images as exported from the gaussian kernel
       plotting function, and plots their addition.
    """
    if xlims is None or xlims is None:
        xlims, ylims = ((0, 100), (0, 100))

    addition = analysis.get_image_add_data(Z1, Z2, normalized=norm)
    fig, ax = cm.get_figure(new_fig=new_fig, subplot=subplot)
    ax.imshow(np.rot90(addition), vmin=vmin, vmax=vmax, cmap=cmap,
              interpolation='bilinear', extent=xlims + ylims)

    kwargs['xlim'] = xlims
    kwargs['ylim'] = ylims
    return cm.plot_style(fig=fig, ax=ax, **kwargs)


def persistence_image_average(ph_list, new_fig=True, subplot=111, xlims=None,
                              ylims=None, norm_factor=1.0, vmin=None, vmax=None,
                              cmap=jet_map, weighted=False, **kwargs):
    """Merges a list of ph diagrams and plots their respective average image.
    """
    # pylint: disable=unexpected-keyword-arg
    av_imgs = analysis.get_average_persistence_image(ph_list, xlims=xlims, ylims=ylims,
                                                     norm_factor=norm_factor, weighted=weighted)
    if xlims is None or xlims is None:
        xlims, ylims = analysis.get_limits(ph_list)

    if vmin is None:
        vmin = np.min(av_imgs)
    if vmax is None:
        vmax = np.max(av_imgs)

    fig, ax = cm.get_figure(new_fig=new_fig, subplot=subplot)
    ax.imshow(np.rot90(av_imgs), vmin=vmin, vmax=vmax, cmap=cmap,
              interpolation='bilinear', extent=xlims + ylims)

    kwargs['xlim'] = xlims
    kwargs['ylim'] = ylims
    kwargs['title'] = kwargs.get('title', 'Average persistence image')
    kwargs['xlabel'] = kwargs.get('xlabel', 'End radial distance from soma')
    kwargs['ylabel'] = kwargs.get('ylabel', 'Start radial distance from soma')

    return av_imgs, cm.plot_style(fig=fig, ax=ax, **kwargs)


def start_length_diagram(ph, new_fig=True, subplot=False, color='b', alpha=1.0, **kwargs):
    '''Plots the transformed ph diagram that represents lengths and starting points
       of a component.
    '''
    ph_transformed = transform_ph_to_length(ph)
    fig, ax = cm.get_figure(new_fig=new_fig, subplot=subplot)

    for p in ph_transformed:
        ax.scatter(p[0], p[1], c=color, edgecolors='black', alpha=alpha)

    kwargs['title'] = kwargs.get('title', 'Transformed Persistence diagram')
    kwargs['xlabel'] = kwargs.get('xlabel', 'Start of the component')
    kwargs['ylabel'] = kwargs.get('ylabel', 'Length of the component')
    return cm.plot_style(fig=fig, ax=ax, **kwargs)


def histogram_stepped(ph, new_fig=True, subplot=False, color='b', alpha=0.7, **kwargs):
    '''Extracts and plots the stepped histogram of a persistent
       homology array.
    '''
    fig, ax = cm.get_figure(new_fig=new_fig, subplot=subplot)
    hist_data = analysis.histogram_stepped(ph)
    ax.fill_between(hist_data[0][:-1], 0, hist_data[1], color=color, alpha=alpha)
    return cm.plot_style(fig=fig, ax=ax, **kwargs)


def histogram_stepped_population(ph_list, new_fig=True, subplot=False,
                                 color='b', alpha=0.7, **kwargs):
    '''Extracts and plots the stepped histogram of a list of persistence diagrams.
       The histogram is normalized acording to the number of persistence diagrams.
    '''
    fig, ax = cm.get_figure(new_fig=new_fig, subplot=subplot)
    hist_data = analysis.histogram_stepped(analysis.collapse(ph_list))
    ax.fill_between(hist_data[0][:-1], 0, hist_data[1] / len(ph_list), color=color, alpha=alpha)
    return cm.plot_style(fig=fig, ax=ax, **kwargs)


def histogram_horizontal(ph, new_fig=True, subplot=False, bins=100, color='b', alpha=0.7, **kwargs):
    '''Extracts and plots the binned histogram of a persistent
       homology array.
    '''
    fig, ax = cm.get_figure(new_fig=new_fig, subplot=subplot)
    hist_data = analysis.histogram_horizontal(ph, num_bins=bins)
    ax.fill_between(hist_data[0][:-1], 0, hist_data[1], color=color, alpha=alpha)

    return cm.plot_style(fig=fig, ax=ax, **kwargs)



'''
Python module that contains the functions
about reading and writing files.
'''
from __future__ import print_function

import os
import numpy as _np
from scipy import sparse as sp
from scipy.sparse import csgraph as cs
from tmd.io.swc import SWC_DCT
from tmd.io.swc import read_swc
from tmd.io.swc import swc_to_data
from tmd.io.h5 import read_h5
from tmd.Neuron import Neuron
from tmd.Tree import Tree
from tmd.Soma import Soma
from tmd.Population import Population
from tmd.utils import tree_type as td

# Definition of tree types
TYPE_DCT = {'soma': 1,
            'basal': 3,
            'apical': 4,
            'axon': 2}


class LoadNeuronError(Exception):
    '''Captures the exception of failing to load a single neuron
    '''


def make_tree(data):
    '''Make tree structure from loaded data.
       Returns a tree of tmd.Tree
       type.
    '''
    tr_data = _np.transpose(data)

    parents = [_np.where(tr_data[0] == i)[0][0]
               if len(_np.where(tr_data[0] == i)[0]) > 0
               else - 1 for i in tr_data[6]]

    return Tree.Tree(x=tr_data[SWC_DCT['x']], y=tr_data[SWC_DCT['y']], z=tr_data[SWC_DCT['z']],
                     d=tr_data[SWC_DCT['radius']], t=tr_data[SWC_DCT['type']], p=parents)


def load_neuron(input_file, line_delimiter='\n', soma_type=None,
                tree_types=None, remove_duplicates=True):
    '''
    Io method to load an swc or h5 file into a Neuron object.
    TODO: Check if tree is connected to soma, otherwise do
    not include it in neuron structure and warn the user
    that there are disconnected components
    '''

    tree_types_final = td.copy()
    if tree_types is not None:
        tree_types_final.update(tree_types)

    # Definition of swc types from type_dict function
    if soma_type is None:
        soma_index = TYPE_DCT['soma']
    else:
        soma_index = soma_type

    # Make neuron with correct filename and load data
    if os.path.splitext(input_file)[-1] == '.swc':
        data = swc_to_data(read_swc(input_file=input_file,
                                    line_delimiter=line_delimiter))
        neuron = Neuron.Neuron(name=input_file.replace('.swc', ''))

    elif os.path.splitext(input_file)[-1] == '.h5':
        data = read_h5(input_file=input_file, remove_duplicates=remove_duplicates)
        neuron = Neuron.Neuron(name=input_file.replace('.h5', ''))

    try:
        soma_ids = _np.where(_np.transpose(data)[1] == soma_index)[0]
    except IndexError:
        raise LoadNeuronError('Soma points not in the expected format')

    # Extract soma information from swc
    soma = Soma.Soma(x=_np.transpose(data)[SWC_DCT['x']][soma_ids],
                     y=_np.transpose(data)[SWC_DCT['y']][soma_ids],
                     z=_np.transpose(data)[SWC_DCT['z']][soma_ids],
                     d=_np.transpose(data)[SWC_DCT['radius']][soma_ids])

    # Save soma in Neuron
    neuron.set_soma(soma)
    p = _np.array(_np.transpose(data)[6], dtype=int) - _np.transpose(data)[0][0]
    # return p, soma_ids
    try:
        dA = sp.csr_matrix((_np.ones(len(p) - len(soma_ids)),
                           (range(len(soma_ids), len(p)),
                            p[len(soma_ids):])), shape=(len(p), len(p)))
    except Exception:
        raise LoadNeuronError('Cannot create connectivity, nodes not connected correctly.')

    # assuming soma points are in the beginning of the file.
    comp = cs.connected_components(dA[len(soma_ids):, len(soma_ids):])

    # Extract trees
    for i in range(comp[0]):
        tree_ids = _np.where(comp[1] == i)[0] + len(soma_ids)
        tree = make_tree(data[tree_ids])
        neuron.append_tree(tree, td=tree_types_final)

    return neuron


def load_population(neurons, tree_types=None, name=None):
    '''Loads all data of recognised format (swc, h5)
       into a Population object.
       Takes as input a directory or a list of files to load.
    '''
    if isinstance(neurons, (list, tuple)):
        files = neurons
        name = name if name is not None else 'Population'
    elif os.path.isdir(neurons):  # Assumes given input is a directory
        files = [os.path.join(neurons, l) for l in os.listdir(neurons)]
        name = name if name is not None else os.path.basename(neurons)

    pop = Population.Population(name=name)

    for filename in files:
        try:
            assert filename.endswith(('.h5', '.swc'))
            pop.append_neuron(load_neuron(filename, tree_types=tree_types))
        except AssertionError:
            raise Warning("{} is not a valid h5 or swc file".format(filename))
        except LoadNeuronError:
            print('File failed to load: {}'.format(filename))

    return pop



'''
tmd class : Neuron
'''
import copy
import numpy as np
from tmd.Soma import Soma
from tmd.Tree import Tree
from tmd.utils import tree_type


class Neuron(object):
    """
    A Neuron object is a container for Trees
    (basal, apical and axon) and a Soma.
    """
    # pylint: disable=import-outside-toplevel
    from tmd.Neuron.methods import size
    from tmd.Neuron.methods import get_bounding_box

    def __init__(self, name='Neuron'):
        '''Creates an empty Neuron object.
        '''
        self.soma = Soma.Soma()
        self.axon = list()
        self.apical = list()
        self.basal = list()
        self.undefined = list()
        self.name = name

    @property
    def neurites(self):
        '''Get neurites'''
        return self.apical + self.axon + self.basal + self.undefined

    @property
    def dendrites(self):
        '''Get dendrites'''
        return self.apical + self.basal

    def rename(self, new_name):
        """
        Modifies the name of the Neuron to new_name.
        """
        self.name = new_name

    def set_soma(self, new_soma):
        """
        If type of object is soma
        sets the neuron soma to new_soma
        """
        if isinstance(new_soma, Soma.Soma):
            self.soma = new_soma

    def append_tree(self, new_tree, td):
        """
        If type of object is tree
        this function finds the type of tree
        and adds the new_tree to the correct
        list of trees in neuron.
        """
        if isinstance(new_tree, Tree.Tree):

            if int(np.median(new_tree.t)) in td.keys():
                neurite_type = td[int(np.median(new_tree.t))]
            else:
                neurite_type = 'undefined'
            getattr(self, neurite_type).append(new_tree)

    def copy_neuron(self):
        """
        Returns a deep copy of the Neuron.
        """
        return copy.deepcopy(self)

    def is_equal(self, neu):
        '''Tests if all neuron structures are the same'''
        eq = np.alltrue([self.soma.is_equal(neu.soma),
                         np.alltrue([t1.is_equal(t2) for t1, t2 in
                                     zip(self.neurites, neu.neurites)])])
        return eq

    def is_same(self, neu):
        '''Tests if all neuron data are the same'''
        eq = np.alltrue([self.name == neu.name,
                         self.soma.is_equal(neu.soma),
                         np.alltrue([t1.is_equal(t2) for t1, t2 in
                                     zip(self.neurites, neu.neurites)])])
        return eq

    def simplify(self):
        '''Creates a copy of itself and simplifies all trees
           to create a skeleton of the neuron
        '''
        neu = Neuron()
        neu.soma = self.soma.copy_soma()

        for tr in self.neurites:
            t = tr.extract_simplified()
            neu.append_tree(t, tree_type)

        return neu



import tmd
import tmd.view as view

pop1 = tmd.io.load_population(directory1)
pop2 = tmd.io.load_population(directory2)

phs1 = [tmd.methods.get_ph_neuron(n, neurite_type='basal') for n in pop1.neurons]
phs2 = [tmd.methods.get_ph_neuron(n, neurite_type='basal') for n in pop2.neurons]

# Normalize the limits
xlims, ylims = tmd.analysis.get_limits(phs1 + phs2)

# Create average images for populations
imgs1 = [tmd.analysis.get_persistence_image_data(p, xlims=xlims, ylims=ylims) for p in phs1]
IMG1 = tmd.analysis.get_average_persistence_image(phs1, xlims=xlims, ylims=ylims)
imgs2 = [tmd.analysis.get_persistence_image_data(p, xlims=xlims, ylims=ylims) for p in phs2]
IMG2 = tmd.analysis.get_average_persistence_image(phs2, xlims=xlims, ylims=ylims)

# You can plot the images if you want to create pretty figures
average_figure1 = view.common.plot_img_basic(IMG1, title='', xlims=xlims, ylims=ylims, cmap=cm.jet)
average_figure2 = view.common.plot_img_basic(IMG2, title='', xlims=xlims, ylims=ylims, cmap=cm.jet)

# Create the diffence between the two images
DIMG = tmd.analysis.get_image_diff_data(IMG1, IMG2) # subtracts IMG2 from IMG1 so anything red IMG1 has more of it and anything blue IMG2 has more of it - or that's how it is supposed to be :)

# Plot the difference between them
diff_image = view.common.plot_img_basic(DIMG, vmin=-1.0, vmax=1.0) # vmin, vmax important to see changes
# Quantify the absolute distance between the two averages
dist = np.sum(np.abs(DIMG))



'''Test tmd.Neuron'''
from nose import tools as nt
import numpy as np
from tmd.Neuron import Neuron
from tmd.io import io
from tmd.Neuron import methods
import os

_path = os.path.dirname(os.path.abspath(__file__))
DATA_PATH = os.path.join(_path, 'data')

# Filenames for testing
sample_file = os.path.join(DATA_PATH, 'sample.swc')



'''Module for viewing neuronal morphologies
'''

import numpy as _np
from matplotlib.collections import LineCollection as _LC
from tmd.view import common as _cm
from tmd import utils as _utils
from tmd.view.common import blues_map


def _get_default(variable, **kwargs):
    '''Returns default variable or kwargs variable if it exists.
    '''
    default = {'linewidth': 1.2,
               'alpha': 0.8,
               'treecolor': None,
               'diameter': True,
               'diameter_scale': 1.0,
               'white_space': 30.}

    return kwargs.get(variable, default[variable])


def trunk(tr, plane='xy', new_fig=True, subplot=False, hadd=0.0, vadd=0.0, N=10, **kwargs):
    '''Generates a 2d figure of the trunk = first N segments of the tree.

    Parameters
    ----------
    tr: Tree
        neurom.Tree object
    '''
    if plane not in ('xy', 'yx', 'xz', 'zx', 'yz', 'zy'):
        return None, 'No such plane found! Please select one of: xy, xz, yx, yz, zx, zy.'

    # Initialization of matplotlib figure and axes.
    fig, ax = _cm.get_figure(new_fig=new_fig, subplot=subplot)

    # Data needed for the viewer: x,y,z,r
    # bounding_box = tr.get_bounding_box()

    def _seg_2d(seg, x_add=0.0, y_add=0.0):

        """2d coordinates required for the plotting of a segment"""

        horz = _utils.term_dict[plane[0]]
        vert = _utils.term_dict[plane[1]]

        horz1 = seg[0][horz] + x_add
        horz2 = seg[1][horz] + x_add
        vert1 = seg[0][vert] + y_add
        vert2 = seg[1][vert] + y_add

        return ((horz1, vert1), (horz2, vert2))

    if len(tr.get_segments()) < N:
        N = len(tr.get_segments())

    segs = [_seg_2d(seg, hadd, vadd) for seg in tr.get_segments()[:N]]

    linewidth = _get_default('linewidth', **kwargs)

    # Definition of the linewidth according to diameter, if diameter is True.

    if _get_default('diameter', **kwargs):

        scale = _get_default('diameter_scale', **kwargs)
        linewidth = [d * scale for d in tr.d]

    treecolor = _cm.get_color(_get_default('treecolor', **kwargs),
                              _utils.tree_type[tr.get_type()])

    # Plot the collection of lines.
    collection = _LC(segs, color=treecolor, linewidth=linewidth,
                     alpha=_get_default('alpha', **kwargs))

    ax.add_collection(collection)

    kwargs['title'] = kwargs.get('title', 'Tree view')
    kwargs['xlabel'] = kwargs.get('xlabel', plane[0])
    kwargs['ylabel'] = kwargs.get('ylabel', plane[1])

    return _cm.plot_style(fig=fig, ax=ax, **kwargs)


def tree(tr, plane='xy', new_fig=True, subplot=False, hadd=0.0, vadd=0.0, **kwargs):
    '''Generates a 2d figure of the tree.

    Parameters
    ----------
    tr: Tree
        neurom.Tree object
    '''
    if plane not in ('xy', 'yx', 'xz', 'zx', 'yz', 'zy'):
        return None, 'No such plane found! Please select one of: xy, xz, yx, yz, zx, zy.'

    # Initialization of matplotlib figure and axes.
    fig, ax = _cm.get_figure(new_fig=new_fig, subplot=subplot)

    # Data needed for the viewer: x,y,z,r
    bounding_box = tr.get_bounding_box()

    def _seg_2d(seg, x_add=0.0, y_add=0.0):

        """2d coordinates required for the plotting of a segment"""

        horz = _utils.term_dict[plane[0]]
        vert = _utils.term_dict[plane[1]]

        horz1 = seg[0][horz] + x_add
        horz2 = seg[1][horz] + x_add
        vert1 = seg[0][vert] + y_add
        vert2 = seg[1][vert] + y_add

        return ((horz1, vert1), (horz2, vert2))

    segs = [_seg_2d(seg, hadd, vadd) for seg in tr.get_segments()]

    linewidth = _get_default('linewidth', **kwargs)

    # Definition of the linewidth according to diameter, if diameter is True.

    if _get_default('diameter', **kwargs):

        scale = _get_default('diameter_scale', **kwargs)
        linewidth = [d * scale for d in tr.d]

    if tr.get_type() not in _utils.tree_type:
        treecolor = 'black'
    else:
        treecolor = _cm.get_color(_get_default('treecolor', **kwargs),
                                  _utils.tree_type[tr.get_type()])

    # Plot the collection of lines.
    collection = _LC(segs, color=treecolor, linewidth=linewidth,
                     alpha=_get_default('alpha', **kwargs))

    ax.add_collection(collection)

    kwargs['title'] = kwargs.get('title', 'Tree view')
    kwargs['xlabel'] = kwargs.get('xlabel', plane[0])
    kwargs['ylabel'] = kwargs.get('ylabel', plane[1])

    white_space = _get_default('white_space', **kwargs)

    kwargs['xlim'] = kwargs.get('xlim', [bounding_box[0][_utils.term_dict[plane[0]]] - white_space,
                                         bounding_box[1][_utils.term_dict[plane[0]]] + white_space])
    kwargs['ylim'] = kwargs.get('ylim', [bounding_box[0][_utils.term_dict[plane[1]]] - white_space,
                                         bounding_box[1][_utils.term_dict[plane[1]]] + white_space])

    return _cm.plot_style(fig=fig, ax=ax, **kwargs)


def soma(sm, plane='xy', new_fig=True, subplot=False, hadd=0.0, vadd=0.0, **kwargs):
    '''Generates a 2d figure of the soma.

    Parameters
    ----------
    soma: Soma
        neurom.Soma object
    '''
    treecolor = kwargs.get('treecolor', None)
    outline = kwargs.get('outline', True)

    if plane not in ('xy', 'yx', 'xz', 'zx', 'yz', 'zy'):
        return None, 'No such plane found! Please select one of: xy, xz, yx, yz, zx, zy.'

    # Initialization of matplotlib figure and axes.
    fig, ax = _cm.get_figure(new_fig=new_fig, subplot=subplot)

    # Definition of the tree color depending on the tree type.
    treecolor = _cm.get_color(treecolor, tree_type='soma')

    # Plot the outline of the soma as a circle, is outline is selected.
    if not outline:
        soma_circle = _cm.plt.Circle(sm.get_center() + [hadd, vadd, 0.0],
                                     sm.get_diameter() / 2., color=treecolor,
                                     alpha=_get_default('alpha', **kwargs))
        ax.add_artist(soma_circle)
    else:
        horz = getattr(sm, plane[0]) + hadd
        vert = getattr(sm, plane[1]) + vadd

        horz = _np.append(horz, horz[0]) + hadd  # To close the loop for a soma viewer.
        vert = _np.append(vert, vert[0]) + vadd  # To close the loop for a soma viewer.

        _cm.plt.plot(horz, vert, color=treecolor,
                     alpha=_get_default('alpha', **kwargs),
                     linewidth=_get_default('linewidth', **kwargs))

    kwargs['title'] = kwargs.get('title', 'Soma view')
    kwargs['xlabel'] = kwargs.get('xlabel', plane[0])
    kwargs['ylabel'] = kwargs.get('ylabel', plane[1])

    return _cm.plot_style(fig=fig, ax=ax, **kwargs)


def neuron(nrn, plane='xy', new_fig=True, subplot=False, hadd=0.0, vadd=0.0,
           neurite_type='all', rotation=None, nosoma=False, new_axes=True, **kwargs):
    '''Generates a 2d figure of the neuron,
    that contains a soma and a list of trees.

    Parameters
    ----------
    neuron: Neuron
        neurom.Neuron object

    Options
    -------
    plane: str
        Accepted values: Any pair of of xyz
        Default value is 'xy'

    linewidth: float
        Defines the linewidth of the tree and soma
        of the neuron, if diameter is set to False.
        Default value is 1.2.

    alpha: float
        Defines the transparency of the neuron.
        0.0 transparent through 1.0 opaque.
        Default value is 0.8.

    treecolor: str or None
        Defines the color of the trees.
        If None the default values will be used,
        depending on the type of tree:
        Soma: "black"
        Basal dendrite: "red"
        Axon : "blue"
        Apical dendrite: "purple"
        Undefined tree: "black"
        Default value is None.

    new_fig: boolean
        Defines if the neuron will be plotted
        in the current figure (False)
        or in a new figure (True)
        Default value is True.

    subplot: matplotlib subplot value or False
        If False the default subplot 111 will be used.
        For any other value a matplotlib subplot
        will be generated.
        Default value is False.

    diameter: boolean
        If True the diameter, scaled with diameter_scale factor,
        will define the width of the tree lines.
        If False use linewidth to select the width of the tree lines.
        Default value is True.

    diameter_scale: float
        Defines the scale factor that will be multiplied
        with the diameter to define the width of the tree line.
        Default value is 1.

    Returns
    --------
    A 3D matplotlib figure with a tree view, at the selected plane.
    '''
    if plane not in ('xy', 'yx', 'xz', 'zx', 'yz', 'zy'):
        return None, 'No such plane found! Please select one of: xy, xz, yx, yz, zx, zy.'

    # Initialization of matplotlib figure and axes.
    fig, ax = _cm.get_figure(new_fig=new_fig, subplot=subplot, new_axes=new_axes)

    kwargs['new_fig'] = False
    kwargs['subplot'] = subplot

    if not nosoma:
        soma(nrn.soma, plane=plane, hadd=hadd, vadd=vadd, **kwargs)

    h = []
    v = []

    to_plot = []

    if rotation == 'apical':
        angle = _np.arctan2(nrn.apical[0].get_pca()[0], nrn.apical[0].get_pca()[1])
        angle = _np.arctan2(rotation[1], rotation[0])

    if neurite_type == 'all':
        to_plot = nrn.neurites
    else:
        for neu_type in neurite_type:
            to_plot = to_plot + getattr(nrn, neu_type)

    for temp_tree in to_plot:
        if rotation is not None:
            temp_tree.rotate_xy(angle)

        bounding_box = temp_tree.get_bounding_box()

        h.append([bounding_box[0][_utils.term_dict[plane[0]]],
                  bounding_box[1][_utils.term_dict[plane[0]]]])
        v.append([bounding_box[0][_utils.term_dict[plane[1]]],
                  bounding_box[1][_utils.term_dict[plane[1]]]])

        tree(temp_tree, hadd=hadd, vadd=vadd, **kwargs)

    kwargs['title'] = kwargs.get('title', nrn.name)
    kwargs['xlabel'] = kwargs.get('xlabel', plane[0])
    kwargs['ylabel'] = kwargs.get('ylabel', plane[1])

    white_space = _get_default('white_space', **kwargs)
    kwargs['xlim'] = kwargs.get('xlim', [_np.min(h) - white_space + hadd,
                                         _np.max(h) + white_space + hadd])
    kwargs['ylim'] = kwargs.get('ylim', [_np.min(v) - white_space + vadd,
                                         _np.max(v) + white_space + vadd])

    return _cm.plot_style(fig=fig, ax=ax, **kwargs)


def all_trunks(nrn, plane='xy', new_fig=True, subplot=False, hadd=0.0, vadd=0.0,
               neurite_type='all', N=10, **kwargs):
    '''Generates a 2d figure of the neuron,
    that contains a soma and a list of trees.

    Parameters
    ----------
    neuron: Neuron
        neurom.Neuron object
    '''
    if plane not in ('xy', 'yx', 'xz', 'zx', 'yz', 'zy'):
        return None, 'No such plane found! Please select one of: xy, xz, yx, yz, zx, zy.'

    # Initialization of matplotlib figure and axes.
    fig, ax = _cm.get_figure(new_fig=new_fig, subplot=subplot)

    kwargs['new_fig'] = False
    kwargs['subplot'] = subplot

    soma(nrn.soma, plane=plane, hadd=hadd, vadd=vadd, **kwargs)

    to_plot = []

    if neurite_type == 'all':
        to_plot = nrn.neurites
    else:
        for neu_type in neurite_type:
            to_plot = to_plot + getattr(nrn, neu_type)

    for temp_tree in to_plot:

        trunk(temp_tree, N=N, **kwargs)

    kwargs['title'] = kwargs.get('title', nrn.name)
    kwargs['xlabel'] = kwargs.get('xlabel', plane[0])
    kwargs['ylabel'] = kwargs.get('ylabel', plane[1])

    kwargs['xlim'] = kwargs.get('xlim', [nrn.soma.get_center()[0] - 2. * N,
                                         nrn.soma.get_center()[0] + 2. * N])

    kwargs['ylim'] = kwargs.get('ylim', [nrn.soma.get_center()[1] - 2. * N,
                                         nrn.soma.get_center()[1] + 2. * N])

    return _cm.plot_style(fig=fig, ax=ax, **kwargs)


def population(pop, plane='xy', new_fig=True, subplot=False, hadd=0.0, vadd=0.0,
               neurite_type='all', **kwargs):
    '''Generates a 2d figure of the population,
    that contains a soma and a list of trees.

    Parameters
    ----------
    pop: Population
        neurom.Population object

    Options
    -------
    plane: str
        Accepted values: Any pair of of xyz
        Default value is 'xy'

    linewidth: float
        Defines the linewidth of the tree and soma
        of the neuron, if diameter is set to False.
        Default value is 1.2.

    alpha: float
        Defines the transparency of the neuron.
        0.0 transparent through 1.0 opaque.
        Default value is 0.8.

    treecolor: str or None
        Defines the color of the trees.
        If None the default values will be used,
        depending on the type of tree:
        Soma: "black"
        Basal dendrite: "red"
        Axon : "blue"
        Apical dendrite: "purple"
        Undefined tree: "black"
        Default value is None.

    new_fig: boolean
        Defines if the neuron will be plotted
        in the current figure (False)
        or in a new figure (True)
        Default value is True.

    subplot: matplotlib subplot value or False
        If False the default subplot 111 will be used.
        For any other value a matplotlib subplot
        will be generated.
        Default value is False.

    diameter: boolean
        If True the diameter, scaled with diameter_scale factor,
        will define the width of the tree lines.
        If False use linewidth to select the width of the tree lines.
        Default value is True.

    diameter_scale: float
        Defines the scale factor that will be multiplied
        with the diameter to define the width of the tree line.
        Default value is 1.

    Returns
    --------
    A 3D matplotlib figure with a tree view, at the selected plane.
    '''
    if plane not in ('xy', 'yx', 'xz', 'zx', 'yz', 'zy'):
        return None, 'No such plane found! Please select one of: xy, xz, yx, yz, zx, zy.'

    # Initialization of matplotlib figure and axes.
    fig, ax = _cm.get_figure(new_fig=new_fig, subplot=subplot)

    kwargs['new_fig'] = False
    kwargs['subplot'] = subplot

    h = []
    v = []

    for nrn in pop.neurons:

        soma(nrn.soma, plane=plane, hadd=hadd, vadd=vadd, **kwargs)

        if neurite_type == 'all':
            neurite_list = ['basal', 'apical', 'axon']
        else:
            neurite_list = [neurite_type]

        for nt in neurite_list:
            for temp_tree in getattr(nrn, nt):

                bounding_box = temp_tree.get_bounding_box()

                h.append([bounding_box[0][_utils.term_dict[plane[0]]] + hadd,
                          bounding_box[1][_utils.term_dict[plane[1]]] + vadd])
                v.append([bounding_box[0][_utils.term_dict[plane[0]]] + hadd,
                          bounding_box[1][_utils.term_dict[plane[1]]] + vadd])

                tree(temp_tree, plane=plane, hadd=hadd, vadd=vadd, **kwargs)

    kwargs['title'] = kwargs.get('title', 'Neuron view')
    kwargs['xlabel'] = kwargs.get('xlabel', plane[0])
    kwargs['ylabel'] = kwargs.get('ylabel', plane[1])
    kwargs['xlim'] = kwargs.get('xlim', [_np.min(h), _np.max(h)])
    kwargs['ylim'] = kwargs.get('ylim', [_np.min(v), _np.max(v)])

    return _cm.plot_style(fig=fig, ax=ax, **kwargs)


def tree3d(tr, new_fig=True, new_axes=True, subplot=False, **kwargs):
    '''Generates a figure of the tree in 3d.

    Parameters
    ----------
    tr: Tree
        neurom.Tree object
    '''
    # pylint: disable=import-outside-toplevel
    from mpl_toolkits.mplot3d.art3d import Line3DCollection

    # Initialization of matplotlib figure and axes.
    fig, ax = _cm.get_figure(new_fig=new_fig, new_axes=new_axes,
                             subplot=subplot, params={'projection': '3d'})

    # Data needed for the viewer: x,y,z,r
    bounding_box = tr.get_bounding_box()

    def _seg_3d(seg):

        """3d coordinates needed for the plotting of a segment"""

        horz = _utils.term_dict['x']
        vert = _utils.term_dict['y']
        depth = _utils.term_dict['z']

        horz1 = seg[0][horz]
        horz2 = seg[1][horz]
        vert1 = seg[0][vert]
        vert2 = seg[1][vert]
        depth1 = seg[0][depth]
        depth2 = seg[1][depth]

        return ((horz1, vert1, depth1), (horz2, vert2, depth2))

    segs = [_seg_3d(seg) for seg in tr.get_segments()]

    linewidth = _get_default('linewidth', **kwargs)

    # Definition of the linewidth according to diameter, if diameter is True.

    if _get_default('diameter', **kwargs):

        scale = _get_default('diameter_scale', **kwargs)
        linewidth = [d * scale for d in tr.d]

    treecolor = _cm.get_color(_get_default('treecolor', **kwargs),
                              _utils.tree_type[tr.get_type()])

    # Plot the collection of lines.
    collection = Line3DCollection(segs, color=treecolor, linewidth=linewidth,
                                  alpha=_get_default('alpha', **kwargs))

    ax.add_collection3d(collection)

    kwargs['title'] = kwargs.get('title', 'Tree 3d-view')
    kwargs['xlabel'] = kwargs.get('xlabel', 'X')
    kwargs['ylabel'] = kwargs.get('ylabel', 'Y')
    kwargs['zlabel'] = kwargs.get('zlabel', 'Z')

    white_space = _get_default('white_space', **kwargs)

    kwargs['xlim'] = kwargs.get('xlim', [bounding_box[0][_utils.term_dict['x']] - white_space,
                                         bounding_box[1][_utils.term_dict['x']] + white_space])
    kwargs['ylim'] = kwargs.get('ylim', [bounding_box[0][_utils.term_dict['y']] - white_space,
                                         bounding_box[1][_utils.term_dict['y']] + white_space])
    kwargs['zlim'] = kwargs.get('zlim', [bounding_box[0][_utils.term_dict['z']] - white_space,
                                         bounding_box[1][_utils.term_dict['z']] + white_space])

    return _cm.plot_style(fig=fig, ax=ax, **kwargs)


def trunk3d(tr, new_fig=True, new_axes=True, subplot=False, N=10, **kwargs):
    '''Generates a figure of the trunk in 3d.

    Parameters
    ----------
    tr: Tree
        neurom.Tree object
    '''
    # pylint: disable=import-outside-toplevel
    from mpl_toolkits.mplot3d.art3d import Line3DCollection

    # Initialization of matplotlib figure and axes.
    fig, ax = _cm.get_figure(new_fig=new_fig, new_axes=new_axes,
                             subplot=subplot, params={'projection': '3d'})

    def _seg_3d(seg):

        """3d coordinates needed for the plotting of a segment"""

        horz = _utils.term_dict['x']
        vert = _utils.term_dict['y']
        depth = _utils.term_dict['z']

        horz1 = seg[0][horz]
        horz2 = seg[1][horz]
        vert1 = seg[0][vert]
        vert2 = seg[1][vert]
        depth1 = seg[0][depth]
        depth2 = seg[1][depth]

        return ((horz1, vert1, depth1), (horz2, vert2, depth2))

    if len(tr.get_segments()) < N:
        N = len(tr.get_segments())

    segs = [_seg_3d(seg) for seg in tr.get_segments()[:N]]

    linewidth = _get_default('linewidth', **kwargs)

    # Definition of the linewidth according to diameter, if diameter is True.

    if _get_default('diameter', **kwargs):

        scale = _get_default('diameter_scale', **kwargs)
        linewidth = [d * scale for d in tr.d]

    treecolor = _cm.get_color(_get_default('treecolor', **kwargs),
                              _utils.tree_type[tr.get_type()])

    # Plot the collection of lines.
    collection = Line3DCollection(segs, color=treecolor, linewidth=linewidth,
                                  alpha=_get_default('alpha', **kwargs))

    ax.add_collection3d(collection)

    kwargs['title'] = kwargs.get('title', 'Tree 3d-view')
    kwargs['xlabel'] = kwargs.get('xlabel', 'X')
    kwargs['ylabel'] = kwargs.get('ylabel', 'Y')
    kwargs['zlabel'] = kwargs.get('zlabel', 'Z')

    return _cm.plot_style(fig=fig, ax=ax, **kwargs)


def soma3d(sm, new_fig=True, new_axes=True, subplot=False, **kwargs):
    '''Generates a 3d figure of the soma.

    Parameters
    ----------
    soma: Soma
        neurom.Soma object
    '''
    treecolor = kwargs.get('treecolor', None)

    # Initialization of matplotlib figure and axes.
    fig, ax = _cm.get_figure(new_fig=new_fig, new_axes=new_axes,
                             subplot=subplot, params={'projection': '3d'})

    # Definition of the tree color depending on the tree type.
    treecolor = _cm.get_color(treecolor, tree_type='soma')

    center = sm.get_center()

    xs = center[0]
    ys = center[1]
    zs = center[2]

    # Plot the soma as a circle.
    fig, ax = _cm.plot_sphere(fig, ax, center=[xs, ys, zs],
                              radius=sm.get_diameter(), color=treecolor,
                              alpha=_get_default('alpha', **kwargs))

    kwargs['title'] = kwargs.get('title', 'Soma view')
    kwargs['xlabel'] = kwargs.get('xlabel', 'X')
    kwargs['ylabel'] = kwargs.get('ylabel', 'Y')
    kwargs['zlabel'] = kwargs.get('zlabel', 'Z')

    return _cm.plot_style(fig=fig, ax=ax, **kwargs)


def neuron3d(nrn, new_fig=True, new_axes=True, subplot=False, neurite_type='all', **kwargs):
    '''Generates a figure of the neuron,
    that contains a soma and a list of trees.

    Parameters
    ----------
    neuron: Neuron
        neurom.Neuron object
    '''
    # Initialization of matplotlib figure and axes.
    fig, ax = _cm.get_figure(new_fig=new_fig, new_axes=new_axes,
                             subplot=subplot, params={'projection': '3d'})

    kwargs['new_fig'] = False
    kwargs['new_axes'] = False
    kwargs['subplot'] = subplot

    soma3d(nrn.soma, **kwargs)

    h = []
    v = []
    d = []

    to_plot = []

    if neurite_type == 'all':
        to_plot = nrn.neurites
    else:
        for neu_type in neurite_type:
            to_plot = to_plot + getattr(nrn, neu_type)

    for temp_tree in to_plot:

        bounding_box = temp_tree.get_bounding_box()

        h.append([bounding_box[0][_utils.term_dict['x']],
                  bounding_box[1][_utils.term_dict['x']]])
        v.append([bounding_box[0][_utils.term_dict['y']],
                  bounding_box[1][_utils.term_dict['y']]])
        d.append([bounding_box[0][_utils.term_dict['z']],
                  bounding_box[1][_utils.term_dict['z']]])

        tree3d(temp_tree, **kwargs)

    kwargs['title'] = kwargs.get('title', nrn.name)
    white_space = _get_default('white_space', **kwargs)
    kwargs['xlim'] = kwargs.get('xlim', [_np.min(h) - white_space,
                                         _np.max(h) + white_space])
    kwargs['ylim'] = kwargs.get('ylim', [_np.min(v) - white_space,
                                         _np.max(v) + white_space])
    kwargs['zlim'] = kwargs.get('zlim', [_np.min(d) - white_space,
                                         _np.max(d) + white_space])

    return _cm.plot_style(fig=fig, ax=ax, **kwargs)


def all_trunks3d(nrn, new_fig=True, new_axes=True, subplot=False, neurite_type='all',
                 N=10, **kwargs):
    '''Generates a figure of the neuron,
    that contains a soma and a list of trees.

    Parameters
    ----------
    neuron: Neuron
        neurom.Neuron object
    '''
    # Initialization of matplotlib figure and axes.
    fig, ax = _cm.get_figure(new_fig=new_fig, new_axes=new_axes,
                             subplot=subplot, params={'projection': '3d'})

    kwargs['new_fig'] = False
    kwargs['new_axes'] = False
    kwargs['subplot'] = subplot

    soma3d(nrn.soma, **kwargs)

    to_plot = []

    if neurite_type == 'all':
        to_plot = nrn.neurites
    else:
        for neu_type in neurite_type:
            to_plot = to_plot + getattr(nrn, neu_type)

    for temp_tree in to_plot:

        trunk3d(temp_tree, N=N, **kwargs)

    kwargs['title'] = kwargs.get('title', nrn.name)
    kwargs['xlim'] = kwargs.get('xlim', [nrn.soma.get_center()[0] - 2. * N,
                                         nrn.soma.get_center()[0] + 2. * N])

    kwargs['ylim'] = kwargs.get('ylim', [nrn.soma.get_center()[1] - 2. * N,
                                         nrn.soma.get_center()[1] + 2. * N])

    kwargs['zlim'] = kwargs.get('zlim', [nrn.soma.get_center()[2] - 2. * N,
                                         nrn.soma.get_center()[2] + 2. * N])

    return _cm.plot_style(fig=fig, ax=ax, **kwargs)


def population3d(pop, new_fig=True, new_axes=True, subplot=False, **kwargs):
    '''Generates a figure of the population,
    that contains the pop somata and a set of list of trees.

    Parameters
    ----------
    pop: Population
        neurom.Population object
    '''
    # Initialization of matplotlib figure and axes.
    fig, ax = _cm.get_figure(new_fig=new_fig, new_axes=new_axes,
                             subplot=subplot, params={'projection': '3d'})

    kwargs['new_fig'] = False
    kwargs['new_axes'] = False
    kwargs['subplot'] = subplot

    h = []
    v = []
    d = []

    for nrn in pop.neurons:
        # soma3d(nrn.soma, **kwargs)
        for temp_tree in nrn.neurites:
            bounding_box = temp_tree.get_bounding_box()
            h.append([bounding_box[0][_utils.term_dict['x']],
                      bounding_box[1][_utils.term_dict['x']]])
            v.append([bounding_box[0][_utils.term_dict['y']],
                      bounding_box[1][_utils.term_dict['y']]])
            d.append([bounding_box[0][_utils.term_dict['z']],
                      bounding_box[1][_utils.term_dict['z']]])
            tree3d(temp_tree, **kwargs)

    kwargs['title'] = kwargs.get('title', '')
    white_space = _get_default('white_space', **kwargs)
    kwargs['xlim'] = kwargs.get('xlim', [_np.min(h) - white_space,
                                         _np.max(h) + white_space])
    kwargs['ylim'] = kwargs.get('ylim', [_np.min(v) - white_space,
                                         _np.max(v) + white_space])
    kwargs['zlim'] = kwargs.get('zlim', [_np.min(d) - white_space,
                                         _np.max(d) + white_space])

    return _cm.plot_style(fig=fig, ax=ax, **kwargs)


# pylint: disable=too-many-locals
def density_cloud(obj, new_fig=True, subplot=111, new_axes=True, neurite_type='all',
                  bins=100, plane='xy', color_map=blues_map, alpha=0.8,
                  centered=True, colorbar=True, plot_neuron=False, **kwargs):
    """
    View the neuron morphologies of a population as a density cloud.
    """
    x1 = []
    y1 = []

    if neurite_type == 'all':
        ntypes = 'neurites'
    else:
        ntypes = neurite_type

    fig, ax = _cm.get_figure(new_fig=new_fig, new_axes=new_axes, subplot=subplot)

    for neu in obj.neurons:
        for tr in getattr(neu, ntypes):
            if centered:
                dx = neu.soma.get_center()[0]
                dy = neu.soma.get_center()[1]
            else:
                dx = 0
                dy = 0
            x1 = x1 + list(getattr(tr, plane[0]) - dx)
            y1 = y1 + list(getattr(tr, plane[1]) - dy)

    H1, xedges1, yedges1 = _np.histogram2d(x1, y1, bins=(bins, bins))
    mask = H1 < 0.05
    H2 = _np.ma.masked_array(H1, mask)
    color_map.set_bad(color='white', alpha=None)

    plots = ax.contourf((xedges1[:-1] + xedges1[1:]) / 2,
                        (yedges1[:-1] + yedges1[1:]) / 2,
                        _np.transpose(H2), cmap=color_map,
                        alhpa=alpha)

    kwargs['new_fig'] = False
    kwargs['subplot'] = subplot

    if plot_neuron:
        nrn = obj.neurons[0]
        h, v, _ = nrn.soma.get_center()
        soma(nrn.soma, plane='xy', hadd=-h, vadd=-v, **kwargs)
        for temp_tree in getattr(nrn, ntypes):
            tree(temp_tree, plane='xy', hadd=-h, vadd=-v, treecolor='r', **kwargs)

    if colorbar:
        _cm.plt.colorbar(plots)
    # soma(neu.soma, new_fig=False)
    return _cm.plot_style(fig=fig, ax=ax, **kwargs)


def _get_polar_data(pop, neurite_type='neurites', bins=20):
    '''Extracts the data to plot the polar length distribution
    of a neuron or a population of neurons.
    '''
    def seg_angle(seg):
        '''angle between mean x, y coordinates of a seg'''
        mean_x = _np.mean([seg[0][0], seg[1][0]])
        mean_y = _np.mean([seg[0][1], seg[1][1]])
        return _np.arctan2(mean_y, mean_x)

    def seg_length(seg):
        '''compute the length of a seg'''
        return _np.linalg.norm(_np.subtract(seg[1], seg[0]))

    segs = []
    for tr in getattr(pop, neurite_type):
        segs = segs + tr.get_segments()

    angles = _np.array([seg_angle(s) for s in segs])
    lens = _np.array([seg_length(s) for s in segs])
    ranges = [[i * 2 * _np.pi / bins - _np.pi, (i + 1) * 2 * _np.pi / bins - _np.pi]
              for i in range(bins)]
    results = [r + [_np.sum(lens[_np.where((angles > r[0]) & (angles < r[1]))[0]])]
               for r in ranges]

    return results


def polar_plot(pop, neurite_type='neurites', bins=20):
    '''
    Generates a polar plot of a neuron or population
    '''
    input_data = _get_polar_data(pop, neurite_type=neurite_type, bins=bins)

    fig = _cm.plt.figure()
    ax = fig.add_axes([0.1, 0.1, 0.8, 0.8], polar=True)

    theta = _np.array(input_data)[:, 0]
    radii = _np.array(input_data)[:, 2] / _np.max(input_data)
    width = 2 * _np.pi / len(input_data)
    ax.bar(theta, radii, width=width, bottom=0.0, alpha=0.8)



'''Test tmd.io.swc'''
import os
from nose import tools as nt
from tmd.io import swc
import numpy as np

_path = os.path.dirname(os.path.abspath(__file__))
DATA_PATH = os.path.join(_path, 'data')

# Filenames for testing
basic_file = os.path.join(DATA_PATH, 'basic.swc')
nocom_file = os.path.join(DATA_PATH, 'basic_no_comments.swc')
nosecids_file = os.path.join(DATA_PATH, 'basic_no_sec_ids.swc')
options_file = os.path.join(DATA_PATH, 'basic_options.swc')
sample_file = os.path.join(DATA_PATH, 'sample.swc')

basic_data = np.array(['1 1 0 0 0 6 -1', '2 2 3 4 5 6 1',
                       '3 3 4 5 6 7 1', '4 4 5 6 7 8 1', '5 4 5 6 7 8 4'])
options_data = np.array(['2', '3', '4', '5', '6', '7', '8', '9\n'])

bx1 = np.array([0.,  3.,  4.,  5.,  5.])
by1 = np.array([0.,  4.,  5.,  6.,  6.])
bz1 = np.array([0.,  5.,  6.,  7.,  7.])
bd1 = np.array([12.,  12.,  14.,  16.,  16.])
bt1 = np.array([1, 2, 3, 4, 4])
bp1 = np.array([-1,  0,  0,  0,  3])
bch1 = {0: [1, 2, 3], 1: [], 2: [], 3: [4], 4: []}


def test_swc_dict():
    nt.ok_(swc.SWC_DCT == {'index': 0,
                           'parent': 6,
                           'radius': 5,
                           'type': 1,
                           'x': 2,
                           'y': 3,
                           'z': 4})


def test_read_swc():
    data1 = swc.read_swc(basic_file)
    nt.ok_(np.alltrue(data1 == basic_data))
    data2 = swc.read_swc(options_file, line_delimiter=' ')
    nt.ok_(np.alltrue(data2 == options_data))


def test_swc_data_to_lists():
    x1, y1, z1, d1, t1, p1, ch1 = swc.swc_data_to_lists(basic_data)
    nt.ok_(np.allclose(x1, bx1))
    nt.ok_(np.allclose(y1, by1))
    nt.ok_(np.allclose(z1, bz1))
    nt.ok_(np.allclose(d1, bd1))
    nt.ok_(np.allclose(t1, bt1))
    nt.ok_(np.allclose(p1, bp1))
    nt.ok_(ch1 == bch1)



'''Test tmd.utils'''
from nose import tools as nt
from tmd import utils

def test_term_dict():
    nt.ok_(utils.term_dict == {'x': 0,
                               'y': 1,
                               'z': 2})

def test_tree_type():
    nt.assert_dict_equal(utils.tree_type,
                         {1: 'soma',
                          2: 'axon',
                          3: 'basal',
                          4: 'apical'})



'''
tmd Neuron's methods
'''
import numpy as np


def size(self, neurite_type='all'):
    """
    Neuron method to get size.
    """
    if neurite_type == 'all':
        neurite_list = ['basal', 'axon', 'apical']

    s = np.sum([len(getattr(self, neu)) for neu in neurite_list])

    return int(s)


def get_bounding_box(self):
    """
    Input
    ------
    neuron: tmd neuron

    Returns
    ---------
    bounding_box: np.array
        ([xmin,ymin,zmin], [xmax,ymax,zmax])
    """
    x = []
    y = []
    z = []

    for tree in self.neurites:
        x = x + tree.x.tolist()
        y = y + tree.y.tolist()
        z = z + tree.z.tolist()

    xmin = np.min(x)
    xmax = np.max(x)
    ymin = np.min(y)
    ymax = np.max(y)
    zmin = np.min(z)
    zmax = np.max(z)

    return np.array([[xmin, ymin, zmin], [xmax, ymax, zmax]])



'''
tmd Tree's methods
'''
import copy
from collections import OrderedDict
import numpy as np
import scipy.sparse as sp
from sklearn.decomposition import PCA


def _rd(point1, point2):
    '''Returns euclidean distance between point1 and point2
    '''
    return np.linalg.norm(np.subtract(point1, point2), 2)


def _rd_w(p1, p2, w=(1., 1., 1.), normed=True):
    '''Returns weighted euclidean distance between p1 and p2
    '''
    if normed:
        w = (np.array(w) / np.linalg.norm(w))
    return np.dot(w, (np.subtract(p1, p2)))


def size(tree):
    '''
    Tree method to get the size of the tree lists.

    Note: All the lists of the Tree should be
    of the same size, but this should be
    checked in the initialization of the Tree!
    '''
    return int(len(tree.x))


def get_type(self):
    '''Returns type of tree
    '''
    return int(np.median(self.t))


def get_bounding_box(self):
    """
    Input
    ------
    tree: tmd tree

    Returns
    ---------
    bounding_box: np.array
        ([xmin,ymin,zmin], [xmax,ymax,zmax])
    """
    xmin = np.min(self.x)
    xmax = np.max(self.x)
    ymin = np.min(self.y)
    ymax = np.max(self.y)
    zmin = np.min(self.z)
    zmax = np.max(self.z)

    return np.array([[xmin, ymin, zmin], [xmax, ymax, zmax]])


# Segment features
def get_segments(self):
    """
    Input
    ------
    tree: tmd tree

    Returns
    ---------
    seg_list: np.array
        (child[x,y,z], parent[x,y,z])
    """
    seg_list = []

    for seg_id in range(1, size(self)):
        par_id = self.p[seg_id]
        child_coords = np.array([self.x[seg_id],
                                 self.y[seg_id],
                                 self.z[seg_id]])
        parent_coords = np.array([self.x[par_id],
                                  self.y[par_id],
                                  self.z[par_id]])
        seg_list.append(np.array([parent_coords, child_coords]))

    return seg_list


def get_segment_lengths(tree):
    '''Returns segment lengths
    '''
    seg_len = np.zeros(size(tree) - 1)
    segs = tree.get_segments()

    for iseg, seg in enumerate(segs):
        seg_len[iseg] = _rd(seg[0], seg[1])

    return seg_len


# Points features to be used for topological extraction
def get_point_radial_distances(self, point=None, dim='xyz'):
    '''Tree method to get radial distances from a point.
    If point is None, the soma surface -defined by
    the initial point of the tree- will be used
    as a reference point.
    '''
    if point is None:
        point = []
        for d in dim:
            point.append(getattr(self, d)[0])

    radial_distances = np.zeros(size(self), dtype=float)

    for i in range(size(self)):
        point_dest = []
        for d in dim:
            point_dest.append(getattr(self, d)[i])

        radial_distances[i] = _rd(point, point_dest)

    return radial_distances


def get_point_radial_distances_time(self, point=None, dim='xyz', zero_time=0, time=1):
    '''Tree method to get radial distances from a point.
    If point is None, the soma surface -defined by
    the initial point of the tree- will be used
    as a reference point.
    '''
    if point is None:
        point = []
        for d in dim:
            point.append(getattr(self, d)[0])
    point.append(zero_time)

    radial_distances = np.zeros(size(self), dtype=float)

    for i in range(size(self)):
        point_dest = []
        for d in dim:
            point_dest.append(getattr(self, d)[i])
        point_dest.append(time)

        radial_distances[i] = _rd(point, point_dest)

    return radial_distances


def get_point_weighted_radial_distances(self, point=None, dim='xyz', w=(1, 1, 1), normed=False):
    '''Tree method to get radial distances from a point.
    If point is None, the soma surface -defined by
    the initial point of the tree- will be used
    as a reference point.
    '''
    if point is None:
        point = []
        for d in dim:
            point.append(getattr(self, d)[0])

    radial_distances = np.zeros(size(self), dtype=float)

    for i in range(size(self)):
        point_dest = []
        for d in dim:
            point_dest.append(getattr(self, d)[i])

        radial_distances[i] = _rd_w(point, point_dest, w, normed)

    return radial_distances


def get_point_path_distances(self):
    '''Tree method to get path distances from the root.
    '''
    seg_len = get_segment_lengths(self)
    path_lengths = np.append(0, copy.deepcopy(seg_len))
    children = get_children(self)

    for i in children.keys():
        path_lengths[children[i]] = path_lengths[children[i]] + path_lengths[i]

    return path_lengths


def get_point_section_lengths(self):
    '''Tree method to get section lengths.
    '''
    lengths = np.zeros(size(self), dtype=float)
    ways, end = self.get_sections_only_points()
    seg_len = get_segment_lengths(self)

    for i, item in enumerate(end):
        lengths[item] = np.sum(seg_len[ways[i]:item])

    return lengths


def get_branch_order(tree, seg_id):
    '''Returns branch order of segment'''
    B = tree.get_multifurcations()
    return sum([1 if i in B else 0 for i in get_way_to_root(tree, seg_id)])


def get_point_section_branch_orders(self):
    '''Tree method to get section lengths.
    '''
    return np.array([get_branch_order(self, i) for i in range(size(self))])


def get_point_projection(self, vect=(0, 1, 0), point=None):
    """Projects each point in the tree (x,y,z) - input_point
       to a selected vector. This gives the orientation of
       each section according to a vector in space, if normalized,
       otherwise it returns the relative length of the section.
    """
    if point is None:
        point = [self.x[0], self.y[0], self.z[0]]

    xyz = np.transpose([self.x, self.y, self.z]) - point

    return np.dot(xyz, vect)


# Section features
def get_sections_2(self):
    '''Tree method to get the sections'
    begining and ending indices.
    '''
    end = np.array(sp.csr_matrix.sum(self.dA, 0) != 1)[0].nonzero()[0]

    if 0 in end:  # If first segment is a bifurcation
        end = end[1:]

    beg = np.append([0], self.p[np.delete(np.hstack([0, 1 + end]), len(end))][1:])

    return beg, end


def get_sections_only_points(self):
    '''Tree method to get the sections'
    begining and ending indices.
    '''
    end = np.array(sp.csr_matrix.sum(self.dA, 0) != 1)[0].nonzero()[0]

    if 0 in end:  # If first segment is a bifurcation
        end = end[1:]

    beg = np.delete(np.hstack([0, 1 + end]), len(end))

    return beg, end


def get_bif_term(self):
    '''Returns number of children per point
    '''
    return np.array(sp.csr_matrix.sum(self.dA, axis=0))[0]


def get_bifurcations(self):
    '''Returns bifurcations
    '''
    bif_term = get_bif_term(self)
    bif = np.where(bif_term == 2.)[0]
    return bif


def get_multifurcations(self):
    '''Returns bifurcations
    '''
    bif_term = get_bif_term(self)
    bif = np.where(bif_term >= 2.)[0]
    return bif


def get_terminations(self):
    '''Returns terminations
    '''
    bif_term = get_bif_term(self)
    term = np.where(bif_term == 0.)[0]
    return term


def get_direction_between(self, start_id=0, end_id=1):
    '''Returns direction of a branch
    defined as end point - start point
    normalized as a unit vector.
    '''
    # pylint: disable=assignment-from-no-return
    vect = np.subtract([self.x[end_id], self.y[end_id], self.z[end_id]],
                       [self.x[start_id], self.y[start_id], self.z[start_id]])

    if np.linalg.norm(vect) != 0.0:
        return vect / np.linalg.norm(vect)
    return vect


def _vec_angle(u, v):
    '''Returns the angle between v and u in 3D.
    '''
    c = np.dot(u, v) / np.linalg.norm(u) / np.linalg.norm(v)
    return np.arccos(c)


def get_angle_between(tree, sec_id1, sec_id2):
    '''Returns local bifurcations angle
    between two sections, defined by their ids.
    sec_id1: the start point of the section #1
    sec_id2: the start point of the section #2
    '''
    beg, end = tree.get_sections_only_points()
    b1 = np.where(beg == sec_id1)[0][0]
    b2 = np.where(beg == sec_id2)[0][0]

    u = tree.get_direction_between(beg[b1],
                                   end[b1])
    v = tree.get_direction_between(beg[b2],
                                   end[b2])

    return _vec_angle(u, v)


def get_way_to_root(tree, sec_id=0):
    '''Returns way to root
    '''
    way = []
    tmp_id = sec_id

    while tmp_id != -1:
        way.append(tree.p[tmp_id])
        tmp_id = tree.p[tmp_id]

    return way


def get_children(tree):
    '''Returns a dictionary of children
       for each node of the tree
    '''
    return OrderedDict({i: np.where(tree.p == i)[0] for i in range(len(tree.p))})


def get_pca(self, plane='xy', component=0):
    '''Returns the i-th principal
    component of PCA on the points
    of the tree in the selected plane
    '''
    pca = PCA(n_components=2)
    pca.fit(np.transpose([getattr(self, plane[0]), getattr(self, plane[1])]))

    return pca.components_[component]
